#
# @(#)$Id: PerlbotCore.pm,v 1.1.1.1 2002/09/03 09:14:56 tolkien Exp $
#

package PerlbotCore;

# This is actually a plugin that contains the core perlbot functionality.
# It's always the first plugin to be loaded.  We need to perform a couple
# of tricks to make it appear like a plugin even though it's not in a subdir
# under the plugins dir.  :)

use strict;
use vars qw($hooks $CONFIG @noload);
use Perlbot;
use Chan;
use pLog;
use Network;

# We could put this in get_hooks but we'd need PerlbotCore:: in front of
# everything... this is a little easier to maintain.
$hooks = {
    'msg'            => \&on_msg,
    'endofmotd'      => \&on_connect,
    'public'         => \&on_public,
    'join'           => \&on_join,
    'part'           => \&on_part,
    'caction'        => \&on_ctcp_action,
    'disconnect'     => \&on_disconnect,
    'cping'          => \&on_ctcp_ping,
    'nicknameinuse'  => \&on_nick_error,
    'nickcollision'  => \&on_nick_error,
    'mode'           => \&on_mode,
    'topic'          => \&on_topic,
    'nick'           => \&on_nick_change,
    'quit'           => \&on_quit,
    'kick'           => \&on_kick,
    'whoisuser'      => \&on_whoisuser,
    'whoischannels'  => \&on_whoischannels,
    'nosucknick'     => \&dump_event

    };


# here's a trick
# ==========
package PerlbotCore::Plugin;

sub get_hooks {
    return $PerlbotCore::hooks;
}
# ==========
# we now return you to your regularly scheduled package

package PerlbotCore;


# ============================================================
# variables
# ============================================================

my $nick_add_char;
my $msglog;

# this is kinda beefy.  look at sub parse_main_config if you're confused.
my %config_handlers =
    (
     chan => sub {
	my $name = to_channel($_[0]->{name}[0]) if $_[0]->{name}[0];
	my $key = '';
	if($_[0]->{key}[0]) { $key = $_[0]->{key}[0]; }
	my $chan = new Chan($name, $_[0]->{flags}[0], $key);
	if($_[0]->{logging}) {
	    $chan->logging($_[0]->{logging}[0]);
	}
	if ($_[0]->{auto_op}) {
	    $chan->auto_op($_[0]->{auto_op}[0]);
	}
	if ($_[0]->{fortune}) {
	    $chan->fortune($_[0]->{fortune}[0]);
	}
	if ($_[0]->{time_on_join}) {
	    $chan->time_on_join($_[0]->{time_on_join}[0]);
	}
	$channels{$name} = $chan;
     },
     server => sub {
	 my $server = $_[0]->{addr}[0];
	 my $port = $_[0]->{port}[0];
	 my $ircname = $_[0]->{ircname}[0];
	 my $net = $_[0]->{net}[0];
	 # if no port specified, default to 6667
	 $port or $port = 6667;
	 print "Adding server: $server:$port...\n" if $debug;
	 push @servers, [$_[0]->{addr}[0], $port, $ircname];
     },
     bot => sub {
	 if($_[0]->{nick}) { @nicks = @{$_[0]->{nick}}; }
	 # if not name given, default to 'perlbot'
	 $nicks[0] or $nicks[0] = 'perlbot';
	 $nick_add_char = $_[0]->{addchar}[0];
	 $nick_add_char or $nick_add_char = '_';
	 $pLog::basedir = $_[0]->{logdir}[0];
	 # if no basedir given, default to the current dir
	 if($dirsep eq '/') {
	   $pLog::basedir or $pLog::basedir = './logs';
	 } else {
	   $pLog::basedir or $pLog::basedir = $dirsep . 'logs';
	 }
	 $plugindir = $_[0]->{plugindir}[0];
	 # if no plugindir given, default to 'plugins'
	 if($dirsep eq '/') {
	   $plugindir or $plugindir = './plugins';
	 } else {
	   $plugindir or $plugindir = $dirsep . 'plugins';
	 }
	 $datadir = $_[0]->{datadir}[0];
	 # if no datadir given, default to 'data'
	 if($dirsep eq '/') {
	   $datadir or $datadir = './data';
	 } else {
	   $datadir or $datadir = $dirsep . 'data';
	 }
	 # grab the list of modules NOT to load
	 if($_[0]->{noload}) { @noload = @{$_[0]->{noload}}; }
     }
     );

# beware: this one is huge!
# This is faster than using some form of switch statement.
# Check out the on_msg sub to find out how we use this beast.
my %command_handlers = 
  (
   join => sub {
     my ($priv_conn, $from, $userhost) = (shift, shift, shift);
     my $chan = shift;
       my $conf = parse_config($CONFIG);
       foreach my $chan_hash (@{$conf->{chan}}) {
	 if ($chan eq $chan_hash->{name}->[0]) {
	   &{$config_handlers{chan}}($chan_hash);
	   $channels{to_channel($chan)}->join($priv_conn);
	   return;
	 }
       }
   },
   reload => sub {
     my ($priv_conn, $from, $userhost) = (shift, shift, shift);
       &parse_main_config;
   },
   load => sub {
     my ($priv_conn, $from, $userhost) = (shift, shift, shift);
     my $newconf = shift;
     if ($newconf) {
	 parse_config($newconf);
     } else {
       $priv_conn->privmsg($from, "Not enough arguments to #load.");
     }
   }
   );


# ============================================================
# event handlers
# ============================================================

sub on_connect {
    my $self = shift;

    foreach (values(%channels)) {
	my $chan = $_->{name};
	print "Joining $chan\n";
	$_->join($self);
    }
}

sub on_msg {
  my ($self, $event) = (shift, shift);
  
  # our commands are signaled by a leading '#'
  if(($event->args)[0] =~ /^\#.*/) {
    # split text (on whitespace) into words
    my @commands = split(';', $event->{args}[0]);
    foreach my $command (@commands) {
      my @params = split (' ', $command);
      # shift off the command and strip the leading #
      my ($tmpcommand) = (shift @params) =~ /^#(.*)/;
      # see if we have a handler for this command
      if (exists($command_handlers{$tmpcommand})) {
	# 3rd param is standard IRC nick!ident@host string
	&{$command_handlers{$tmpcommand}}($self, $event->nick, $event->nick.'!'.$event->userhost, @params);
      }
    }
  }
}

sub on_public {
  my ($self, $event) = (shift, shift);
  my $nick = $event->nick;
  my $chan = ($event->to)[0];
  my $text = ($event->args)[0];
  
  if(exists($channels{to_channel($chan)})) {
    $channels{to_channel($chan)}->log_write("<$nick> $text");
  }
}

sub on_join {
  my ($self, $event) = @_;
  my $nick = $event->nick;
  my $chan = ($event->to)[0]; #don't ask me, yo...
  
  if(exists($channels{to_channel($chan)})) {
    $channels{to_channel($chan)}->log_write("$nick joined $chan");
  }
}

sub on_part {
  my ($self, $event) = @_;
  my $nick = $event->nick;
  my $chan = ($event->to)[0];
  
  if(exists($channels{to_channel($chan)})) {
    $channels{to_channel($chan)}->log_write("$nick left $chan");
  }
}

sub on_ctcp_action {
  my ($self, $event) = @_;
  my $nick = $event->nick;
  my $chan = ($event->to)[0];
  shift @{$event->{args}};
  my $text = join(' ', @{$event->{args}});
  
  if(exists($channels{to_channel($chan)})) {
    $channels{to_channel($chan)}->log_write("$nick $text");
  }
}

sub on_topic {
  my $self = shift;
  my $event = shift;
  my $nick = $event->nick;
  my $chan = $event->{to}[0];

  if(exists($channels{to_channel($chan)})) {
    $channels{to_channel($chan)}->log_write("[TOPIC] $nick: " . $event->{args}[0]);
  }
}

sub on_nick_change {
  my $self = shift;
  my $event = shift;
  my $nick = $event->{args}[0]; #this is their NEW nick...

  foreach (values(%channels)) {
    if ($_->{logging} eq "on") {
      my $chan = $_->{name};
      $channels{to_channel($chan)}->log_write("[NICK] " . $event->nick . " changed nick to: $nick");
    }
  }
}

sub on_quit {
  my $self = shift;
  my $event = shift;
  my $nick = $event->nick;

  foreach (values(%channels)) {
    if ($_->{logging} eq "on") {
      my $chan = $_->{name};
      $channels{to_channel($chan)}->log_write("[QUIT] $nick quit: " . $event->{args}[0]);
    }
  }
}

sub on_kick {
  my $self = shift;
  my $event = shift;
  my $nick = $event->nick;
  my $who = $event->{to}[0];
  my $chan = $event->{args}[0];

  if(exists($channels{to_channel($chan)})) {
    $channels{to_channel($chan)}->log_write("[KICK] $who was kicked by $nick (" . $event->{args}[1] . ')');
  }
}

sub on_whoisuser {
  my $self = shift;
  my $event = shift;
  my $nick = $event->{args}[0];
  my $host = join('@', $event->{args}[2], $event->{args}[3]);

  # do nothing...
}

sub on_whoischannels {
  my $self = shift;
  my $event = shift;
  my $nick = $event->{args}[1];
  my $user = '';

  # do nothing...
}

sub on_disconnect {
    my ($self, $event) = @_;
    my $old_server = $event->{from};
    my $server;
    my $i;

    if($$ == 0) {    # exit if we're a child...
      exit;
    }

    if($debug) {
	print "Disconnected from: $old_server\n";
	$event->dump();
	print "---End dump...\n";
    }
    
    while(!$self->connected()) {
	for($i = 0; $i<@servers; $i++) {
	    if($servers[$i]->[0] eq $old_server) { $i++; last; }
	    if($i == @servers - 1) { $i = 0; $old_server = ''; last; }
	}

	for($i; $i<@servers; $i++) {
	    $server = join(':', $servers[$i]->[0], $servers[$i]->[1]);
            print "trying $server\n" if $debug;
            $self->server($server);
            $self->connect();
            if($self->connected()) {
                return;
            }
	    if($i == @servers - 1) { $i = 0; next; }
	}

	print "Sleeping for 10 seconds...\n" if $debug;
	sleep(10);
	$i = 0;
	$old_server = '';
    }
}
	
sub on_ctcp_ping {
    my ($self, $event) = @_;
    my $nick = $event->nick;

    $self->ctcp_reply($nick, join (' ', ($event->args)));
}

sub on_nick_error {
    my ($self, $event) = @_;
    my $use_this_one = 0;

    print "nick error...\n" if $debug;

    if(@nicks < 2) {
	$nicks[0] = $nicks[0] . $nick_add_char;
	$self->nick($nicks[0]);
    } else {
	foreach(@nicks) {
	    if($use_this_one) {
		$self->nick($_);
		return;
	    } else {
		if($_ eq $self->nick()) {
		    $use_this_one = 1;
		    next;
		}
	    }
	}
	
	# if every nickname we like is in use... we do this...
	$nicks[0] = $nicks[0] . $nick_add_char;
	$self->nick($nicks[0]);
    }
}

sub on_mode {
  my $conn = shift;
  my $event = shift;
  my $chan = $event->{to}[0];

  if(exists($channels{to_channel($chan)})) {
    $channels{to_channel($chan)}->log_write("[MODE] " . $event->nick . ' set mode: ' . $event->{args}[0] . ' ' . $event->{args}[1]);
  }

  if(($event->{args}[1] eq $conn->nick) && ($event->{args}[0] =~ /\+o/)) {
    if(exists($channels{to_channel($chan)})) {
      $conn->mode(to_channel($chan), $channels{to_channel($chan)}->{flags});
    }
  }
}

# ============================================================
# other misc utility subs
# ============================================================

# Params:
#   1) file name for main config file
# Returns:
#   If there was some error : an error string
#   If there was no error   : undef
sub parse_main_config {
    my $ret = parse_config($CONFIG);
    # If $ret isn't a hash ref, it's an error string... :(
    if (ref($ret) ne 'HASH') {
	print $ret, "\n";
	exit(1);
    }

    # This is less robust that iterating over keys(%$ret).  However,
    # it removes any ordering requirements in the config file, AND it
    # fixes what we broke by originally using keys(%$ret) (namely everything
    # to do with users and channels).
    # -> Special thanks to Christian Mogensen <christian@superoffice.no> for
    #    the nicer fix to this... (putting the strings in an array, instead
    #    of having a separate 'foreach my $fields' loop for each string :)
    foreach my $class ('bot','server','chan') {
	foreach my $fields (@{$ret->{$class}}) {
	    &{$config_handlers{$class}}($fields);
	}
    }

    # Die if critical things haven't been set, and warn if other important
    # but non-critical things haven't been set.
    %channels or print "\nWARNING: No channels specified.  Is this really what you want?\n";
    @servers or print "\n\nFATAL: No servers specified.  You need at least one server.\n", exit(1);
}

# this just takes an event and dumps it to stdout...

sub dump_event {
  my $self = shift;
  my $event = shift;

  $event->dump();
}

1;
