/*--+
 |
 |  key2code.c
 |
 |  key2code - key stroke -> hangul code converter
 |
 |  written by perky, 2000.
 |  Copyright(C) HanIRC FreeBSD Project Group.
 |
 */
 
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <err.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/time.h>
#include <sys/types.h>	/* size_t, ssize_t */
#include <stdio.h>	/* FILE */
#include <iconv.h>	/* iconv_t */

typedef ssize_t (*iconv_stream_func)(void *d, void *buf, size_t nbytes);

typedef struct {
	iconv_t		cd;
	size_t		chars;
	size_t		in_bytes;
	size_t		out_bytes;
	char *		buffer;
	char *		buf_ptr;
	void *		handle;
	iconv_stream_func method;
} iconv_stream;

iconv_stream *iconv_stream_open(iconv_t cd, void *handle,
                                iconv_stream_func method);
void iconv_stream_close(iconv_stream *stream);

iconv_stream *iconv_ostream_fopen(iconv_t cd, FILE *handle);

ssize_t iconv_write(void *stream, const void *buf, size_t nbytes);
ssize_t iconv_bwrite(void *stream, const void *buf, size_t nbytes);

#define BUF_SIZE 4096

#include "key2code.h"

#include "keycode.def"

const char *codename_johap="johab", *codename_wansung="euc-kr";

char topcharofcase[2]={'a', 'A'};

#define assign_han(han, str)    { (han).ascii[1]=*(str); (han).ascii[0]=*((str)+1); }
#define retrieve_han(han, str)  { *(str)=(han).ascii[1]; *((str)+1)=(han).ascii[0]; }
#define isjaeum(casen, ch)      (key2tbl[casen][ch-topcharofcase[casen]].chosung|key2tbl[casen][ch-topcharofcase[casen]].jongsung ? TRUE : FALSE)
#define ismoeum(casen, ch)      (key2tbl[casen][ch-topcharofcase[casen]].jungsung ? TRUE : FALSE)
#define havechosung(casen, ch)  (key2tbl[casen][ch-topcharofcase[casen]].chosung ? TRUE : FALSE)
#define havejungsung(casen, ch) (key2tbl[casen][ch-topcharofcase[casen]].jungsung ? TRUE : FALSE)
#define havejongsung(casen, ch) (key2tbl[casen][ch-topcharofcase[casen]].jongsung ? TRUE : FALSE)
#define getchosung(casen, ch)   (key2tbl[casen][ch-topcharofcase[casen]].chosung)
#define getjungsung(casen, ch)  (key2tbl[casen][ch-topcharofcase[casen]].jungsung)
#define getjongsung(casen, ch)  (key2tbl[casen][ch-topcharofcase[casen]].jongsung)
#define autofill(ch)            { (ch).bitf.chosung=(ch).bitf.chosung?(ch).bitf.chosung:1; \
                                  (ch).bitf.jungsung=(ch).bitf.jungsung?(ch).bitf.jungsung:2; \
                                  (ch).bitf.jongsung=(ch).bitf.jongsung?(ch).bitf.jongsung:1; }
#define getcasenum(casen, ch)   ((ch)-topcharofcase[(casen)])
#define getcasetbln(ch)         (islower(ch) ? 0 : 1)

int
han2_cnvt(iconv_stream *is, char *str)
{
    unsigned char mergebuff[3], charbuff[2]={NULL,NULL};
    unsigned char wordbuff[BUF_SIZE], alphabuff[BUF_SIZE];
    union UN_HANGUL ch;
    char *trace=str;
    int casen, i, found;
    int validhangul;

    ch.ascii[0]=ch.ascii[1]=NULL;
    mergebuff[2]=NULL;
    *alphabuff=*wordbuff=NULL;
    validhangul=TRUE;

    #define pushhan() { checkvalid(ch); autofill(ch); retrieve_han(ch, mergebuff); strcat((char*)wordbuff, (char*)mergebuff); ch.ascii[0]=ch.ascii[1]=NULL; }
    #define checkvalid(ch) { if(!ch.bitf.chosung || !ch.bitf.jungsung) validhangul=FALSE; }
    #define checkMSB() ch.bitf.MSB=1

    while(*trace) {
        if(isalpha(*trace)) {
            *charbuff=*trace;
            strcat((char*)alphabuff, (char*)charbuff);
            casen=getcasetbln(*trace);  // table num for key2tbl
            switch(isjaeum(casen,*trace)) {
            case TRUE:  // jaeum
                if(!ch.bitf.chosung) {
                    if(ch.bitf.jungsung || ch.bitf.jongsung) 
                        pushhan();
                    checkMSB();
                    ch.bitf.chosung=getchosung(casen, *trace);
                } else if(!ch.bitf.jungsung) { // chosung O jungsung X 
                    pushhan();
                    checkMSB();
                    ch.bitf.chosung=getchosung(casen, *trace);
                } else if(!ch.bitf.jongsung) { // chosung O jungsung O jongsung X
                    checkMSB();
                    if(!(ch.bitf.jongsung=getjongsung(casen, *trace))) { // no jongsung jaeum
                        pushhan();
                        checkMSB();
                        ch.bitf.chosung=getchosung(casen, *trace);
                    }
                } else { // chosung O jungsung O jongsung O
                    found=FALSE;
                    for(i=0; multi2tbl[getcasenum(getcasetbln(*(trace-1)), *(trace-1))][i].orgcode[0]; i++) // must trace>str!
                        if(multi2tbl[getcasenum(getcasetbln(*(trace-1)), *(trace-1))][i].orgcode[1]==*trace &&
                            !(isalpha(*(trace+1)) && ismoeum(getcasetbln(*(trace+1)), *(trace+1)))) {
                            found=TRUE;
                            break;
                        }
                    if(found) { // if multiple jongsung
                        checkMSB();
                        ch.bitf.jongsung=multi2tbl[getcasenum(getcasetbln(*(trace-1)), *(trace-1))][i].jongsung;
                        pushhan();
                    } else {
                        pushhan();
                        checkMSB();
                        ch.bitf.chosung=getchosung(casen, *trace);
                    }
                }
                break;
            case FALSE: // moeum
                if(!ch.bitf.jongsung) {
                    if(!ch.bitf.jungsung) { // jungsung X jongsung X
                        checkMSB();
                        ch.bitf.jungsung=getjungsung(casen, *trace);
                    } else {    // jungsung O jongsung X
                        found=FALSE;
                        for(i=0; multi2tbl[getcasenum(getcasetbln(*(trace-1)), *(trace-1))][i].orgcode[0]; i++) // must trace>str!
                            if(multi2tbl[getcasenum(getcasetbln(*(trace-1)), *(trace-1))][i].orgcode[1]==*trace) {
                                found=TRUE;
                                break;
                            }
                        if(found) { // if multiple jungsung
                            checkMSB();
                            ch.bitf.jungsung=multi2tbl[getcasenum(getcasetbln(*(trace-1)), *(trace-1))][i].jungsung;
                        } else {
                            pushhan();
                            checkMSB();
                            ch.bitf.jungsung=getjungsung(casen, *trace);
                        }
                    }
                } else {    // jongsung O (must trace>str)
                    ch.bitf.jongsung=NULL;
                    pushhan();
                    checkMSB();
                    ch.bitf.chosung=getchosung(getcasetbln(*(trace-1)), *(trace-1));
                    ch.bitf.jungsung=getjungsung(casen, *trace);
                }
                break;
            }
        } else {
            if(ch.bitf.MSB) {// something in buffer
                pushhan();
                if(validhangul) 
                    iconv_write(is, wordbuff, strlen((char*)wordbuff));
                else fprintf(stdout, (char*)alphabuff);
                
                *wordbuff=NULL;
                *alphabuff=NULL;
            }
            fputc(*trace, stdout);
            validhangul=TRUE;
        }
        trace++;
    }
    
    if(ch.bitf.MSB)
        pushhan();

    if(validhangul) 
        iconv_write(is, wordbuff, strlen((char*)wordbuff));
    else fprintf(stdout, (char*)alphabuff);
    
    return TRUE;
}

int
hanroman_cnvt(iconv_stream *is, int method, char *str)
{
	char charbuff[3], outbuffer[8];
	char *inbuf=charbuff, *outbuf=outbuffer;
	char letterbuf[32];
	size_t insize, outsize;
	int wordtopchar;
    union UN_HANGUL ch;
    FILE *fp=(FILE*)(is->handle);

	wordtopchar=TRUE;
	
	while(*str) {
		if(*str<0) {
			charbuff[0]=*(str++);
			if(*str) charbuff[1]=*(str++);
			else continue;
			
			insize=sizeof(charbuff);	outsize=sizeof(outbuffer);
			inbuf=charbuff;				outbuf=outbuffer;
			if(iconv(is->cd, (const char**)&inbuf, &insize, &outbuf, &outsize)<0)
				continue;
				
			assign_han(ch, outbuffer);
			*letterbuf=NULL;
			if(ch.bitf.chosung>1) strcat(letterbuf, han2romantbl[method][ch.bitf.chosung][0]);
			if(ch.bitf.jungsung>2) strcat(letterbuf, han2romantbl[method][ch.bitf.jungsung][1]);
			if(ch.bitf.jongsung>1) strcat(letterbuf, han2romantbl[method][ch.bitf.jongsung][2]);
			
			if(*letterbuf) {
				if(method==0) {
					if(wordtopchar) {
						wordtopchar=FALSE;
						*letterbuf=toupper(*letterbuf);
					} else 
						fputc('-', fp);
				}

				fprintf(fp, letterbuf);
			}
		} else {
			fputc(*(str++), fp);
			wordtopchar=TRUE;
		}
	}
	
	return 1;
}

int
main(int argc, char * const *argv)
{
    iconv_t cd;
    iconv_stream *is;
    char input[BUF_SIZE];
    int i;

	if(strstr(argv[0], "keyecode")) {
	    cd = iconv_open(codename_johap, codename_wansung);
	    if ((int)cd < 0)
	        err(6, "unable to open specified convertor");
	    if (!(is = iconv_ostream_fopen(cd, stdout))) {
	        iconv_close(cd);
	        exit(7);
	    }
	
	    if(argc<2) {
	        while(!feof(stdin)) {
	            *input=NULL;
	            fgets(input, BUF_SIZE, stdin);
	            
	            if (hanroman_cnvt(is, 0, input) <= 0) {
	                exit(8);
	            }
	        }
	    } else {
	        for(i=0; i<argc-1; i++) {
	            strncpy(input, argv[i+1], BUF_SIZE);
	            if (hanroman_cnvt(is, 0, input) <= 0) {
	                exit(8);
	            }
	            fprintf(stdout, "\n");
	        }
	    }
	
	    if (iconv_write(is, NULL, 0) < 0)
	        exit(9);
	    iconv_stream_close(is);
	    iconv_close(cd);
	} else if(strstr(argv[0], "code2key")) {
	    cd = iconv_open(codename_johap, codename_wansung);
	    if ((int)cd < 0)
	        err(6, "unable to open specified convertor");
	    if (!(is = iconv_ostream_fopen(cd, stdout))) {
	        iconv_close(cd);
	        exit(7);
	    }
	
	    if(argc<2) {
	        while(!feof(stdin)) {
	            *input=NULL;
	            fgets(input, BUF_SIZE, stdin);
	            
	            if (hanroman_cnvt(is, 1, input) <= 0) {
	                exit(8);
	            }
	        }
	    } else {
	        for(i=0; i<argc-1; i++) {
	            strncpy(input, argv[i+1], BUF_SIZE);
	            if (hanroman_cnvt(is, 1, input) <= 0) {
	                exit(8);
	            }
	            fprintf(stdout, "\n");
	        }
	    }
	
	    if (iconv_write(is, NULL, 0) < 0)
	        exit(9);
	    iconv_stream_close(is);
	    iconv_close(cd);
	} else {
	    cd = iconv_open(codename_wansung, codename_johap);
	    if ((int)cd < 0)
	        err(6, "unable to open specified convertor");
	    if (!(is = iconv_ostream_fopen(cd, stdout))) {
	        iconv_close(cd);
	        exit(7);
	    }
	
	    if(argc<2) {
	        while(!feof(stdin)) {
	            *input=NULL;
	            fgets(input, BUF_SIZE, stdin);
	            
	            if (han2_cnvt(is, input) <= 0) {
	                exit(8);
	            }
	        }
	    } else {
	        for(i=0; i<argc-1; i++) {
	            strncpy(input, argv[i+1], BUF_SIZE);
	            if (han2_cnvt(is, input) <= 0) {
	                exit(8);
	            }
	            fprintf(stdout, "\n");
	        }
	    }
	
	    if (iconv_write(is, NULL, 0) < 0)
	        exit(9);
	    iconv_stream_close(is);
	    iconv_close(cd);
	}
    return 0;
}
