#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "sol2lunar.h"

#define MIDSTR_MAX	128
#define LONGSTR_MAX	512
#define INVALID		-1

#define RANKLIMIT	5
#define CLOSEBIRTH	7
#define YEARMAX		100

#define ISIN(a, b, c)				((b)>=(a) && (b)<=(c))
#define PSEUDO_YDAY(mon, day)		((mon)*31+(day))

char *weekn[7]={"", "", "ȭ", "", "", "", ""};

struct TZodiac {
	char *name;
	int min_day, max_day;
} zodiac[]={
	{ "õĪ", 		PSEUDO_YDAY( 9, 23), PSEUDO_YDAY( 10, 21) },
	{ "", 		PSEUDO_YDAY(10, 22), PSEUDO_YDAY( 11, 21) },
	{ "", 		PSEUDO_YDAY(11, 22), PSEUDO_YDAY( 12, 21) },
	{ "", 		PSEUDO_YDAY(12, 22), PSEUDO_YDAY( 12, 31) },
	{ "", 		PSEUDO_YDAY( 1,  1), PSEUDO_YDAY(  1, 19) },
	{ "", 		PSEUDO_YDAY( 1, 20), PSEUDO_YDAY(  2, 18) },
	{ "", 	PSEUDO_YDAY( 2, 19), PSEUDO_YDAY(  3, 20) },
	{ "", 		PSEUDO_YDAY( 3, 21), PSEUDO_YDAY(  4, 20) },
	{ "Ȳ", 		PSEUDO_YDAY( 4, 21), PSEUDO_YDAY(  5, 20) },
	{ "ֵ", 	PSEUDO_YDAY( 5, 21), PSEUDO_YDAY(  6, 21) },
	{ "", 		PSEUDO_YDAY( 6, 22), PSEUDO_YDAY(  7, 22) },
	{ "", 		PSEUDO_YDAY( 7, 23), PSEUDO_YDAY(  8, 22) },
	{ "ó", 		PSEUDO_YDAY( 8, 23), PSEUDO_YDAY(  9, 22) },
	{ NULL, NULL, NULL }
};

char *twelves[]={
	"", "", "ȣ", "䳢", "", "", "", "", "", "", "", ""
};

#define ERR_TRANS_RADIX_INVALID		-1
#define ERR_TRANS_RADIX_OK			 0

int
trans_radix(int value, char *str, int len, int radixdst)
{
	#define TOASCII(num)	(((num)>=0 && (num)<=9) ? ((num)+'0') : ((num)+'A'))
	register char *trc;
	
	for(trc=str+len-2, *(trc+1)=NULL; trc>=str; --trc) {
		*trc=TOASCII(value%radixdst);
		if(!(value/=radixdst)) break;
	}
	
	if(trc<str)
		return ERR_TRANS_RADIX_INVALID;
	else if(trc==str)
		return ERR_TRANS_RADIX_OK;
	else {
		while(*trc) 
			*(str++)=*(trc++);
		*str=NULL;
		return ERR_TRANS_RADIX_OK;
	}
	
	#undef TOASCII
}

void
printagegrp(char *args)
{
	FILE *fp;
	int count;
	char linebuffer[LONGSTR_MAX], filename[LONGSTR_MAX];
	int varyr, varmn, varday, ageyr, nmonth, monthcnt, thiscount;
	char birthgain;
	char strtmp[LONGSTR_MAX];

	sscanf(args, "%s %d", filename, &ageyr);

	fp=fopen(filename, "rt");
	if(!fp) return;
	
	if(ageyr>100) ageyr-=1900;
	else if(ageyr<20) ageyr+=100;

	count=0;
	nmonth=0;
	monthcnt=0;
	thiscount=0;
	
	while(!feof(fp)) {
		if(!fgets(linebuffer, LONGSTR_MAX, fp)) 
			continue;
		if(*linebuffer=='#') break;
		
		*strtmp=NULL;
		
		sscanf(linebuffer, "%d %d %d %c %s", &varyr, &varmn, &varday, &birthgain, strtmp);
		
		if(!*strtmp) continue;
		
		if(varyr>100) varyr-=1900;
		else if(varyr<20) varyr+=100;
		
		if(ageyr==varyr) {
			if(nmonth!=varmn) {
				if(monthcnt>0) printf("}, ");
				printf("{%d:%s", varmn, strtmp);
				
				monthcnt=1;
			} else {
				printf(", %s", strtmp);
				
				monthcnt++;
			}
			nmonth=varmn;
			thiscount++;
		}
		
		count++;
	}
	
	if(monthcnt) printf("} -- ü %d %d (%.1f%%)\n", count, thiscount, 100.0*thiscount/count);
	else printf("  ̱~\n");
	
	fclose(fp);
}

void
printleftbirth(char *args)
{
	int varyr, varmn, varday, diffday;
	char birthgain;
	char strtmp[MIDSTR_MAX];

	time_t ntime, btime;
	struct tm ntm, btm, ltm, sltm;
	
	*strtmp=NULL;
	birthgain=NULL;
	varyr=varmn=NULL;
	varday=1;
	
	sscanf(args, "%d %d %d %c %s", &varyr, &varmn, &varday, &birthgain, strtmp);
	if(!birthgain) return;
	if(varyr>100) varyr-=1900;
	else if(varyr<20) varyr+=100;

	time(&ntime);
	localtime_r(&ntime, &ntm);
	ntm.tm_sec=0;
	ntm.tm_min=0;
	ntm.tm_hour=0;
	ntime=mktime(&ntm);
	localtime_r(&ntime, &ntm);
	
	if(birthgain!='-') {
		btm.tm_sec=0;
		btm.tm_min=0;
		btm.tm_hour=0;
		btm.tm_mday=varday;
		btm.tm_mon=varmn-1;
		
		btm.tm_year=ntm.tm_year;

		if(PSEUDO_YDAY(varmn, varday)<PSEUDO_YDAY(ntm.tm_mon+1, ntm.tm_mday))
			btm.tm_year++;
		
		btime=mktime(&btm);
		localtime_r(&btime, &btm);

		diffday=difftime(btime, ntime)/60/60/24;
		
		if(diffday) 
			printf("%s  ϱ %d ҽϴ.\n", strtmp, diffday);
		else
			printf(" մϴ~ ϴ %s~~~  մϴ~~~\n", strtmp);
	} else {
		sol2lunar(&sltm, &ntm);
		/*printf(" ³¥ : %d %d %d\n", sltm.tm_year+1900, sltm.tm_mon+1, sltm.tm_mday);*/
		
		btm.tm_sec=0;
		btm.tm_min=0;
		btm.tm_hour=0;
		btm.tm_mday=varday;
		btm.tm_mon=varmn-1;
		btm.tm_year=varyr;
		
		if(sol2lunar(&ltm, &btm)) {
			printf(" ȯ  ߻߽ϴ.\n");
			return;
		}
		/*printf(" ³¥ : %d %d %d\n", btm.tm_year+1900, btm.tm_mon+1, btm.tm_mday);
		printf(" ³¥ : %d %d %d\n", ltm.tm_year+1900, ltm.tm_mon+1, ltm.tm_mday);*/
		
		ltm.tm_year=sltm.tm_year;
		if(PSEUDO_YDAY(sltm.tm_mon, sltm.tm_mday)>PSEUDO_YDAY(ltm.tm_mon, ltm.tm_mday))
			ltm.tm_year++;

		/*printf("  ³¥ : %d %d %d\n", ltm.tm_year+1900, ltm.tm_mon+1, ltm.tm_mday);*/
		
		if(lunar2sol(&btm, &ltm)) {
			printf(" ȯ  ߻߽ϴ.\n");
			return;
		}
		/*printf("  ³¥ : %d %d %d\n", btm.tm_year+1900, btm.tm_mon+1, btm.tm_mday);*/
		
		btime=mktime(&btm);
		localtime_r(&btime, &btm);
		
		diffday=difftime(btime, ntime)/60/60/24;
		
		if(diffday) 
			printf("%s  ϱ %d ҽϴ. ( %d %d %dԴϴ.)\n", strtmp, diffday,
								btm.tm_year+1900, btm.tm_mon+1, btm.tm_mday);
		else
			printf(" մϴ~ ϴ %s~~~  մϴ~~~\n", strtmp);
	}
}

void
printclosebirth(char *filename)
{
	FILE *fp;
	int count;
	char linebuffer[LONGSTR_MAX];
	int varyr, varmn, varday, diffday, lunaryear;
	char birthgain;
	char strtmp[MIDSTR_MAX];

	time_t ntime, btime, sltime;
	struct tm ntm, btm, ltm, sltm;
	
	time(&ntime);
	localtime_r(&ntime, &ntm);
	ntm.tm_sec=0;
	ntm.tm_min=0;
	ntm.tm_hour=0;
	ntime=mktime(&ntm);
	localtime_r(&ntime, &ntm);
	
	sol2lunar(&ltm, &ntm);
	lunaryear=ltm.tm_year;
	
	fp=fopen(filename, "rt");
	if(!fp) return;
	
	count=0;
	
	while(!feof(fp)) {
		if(!fgets(linebuffer, LONGSTR_MAX, fp)) 
			continue;
		if(*linebuffer=='#' || count>=5) break;
		
		sscanf(linebuffer, "%d %d %d %c %s", &varyr, &varmn, &varday, &birthgain, strtmp);
		
		btm.tm_sec=0;
		btm.tm_min=0;
		btm.tm_hour=0;
		btm.tm_mday=varday;
		btm.tm_mon=varmn-1;
		btm.tm_year=ntm.tm_year;
		
		btime=mktime(&btm);
		localtime_r(&btime, &btm);
		
		if(birthgain=='+' || birthgain=='.') {
			diffday=difftime(btime, ntime)/60/60/24;
			
			if(diffday>0 && diffday<CLOSEBIRTH) {
				printf("%2d %02d (%s)  %s ()\n", varmn, varday, weekn[btm.tm_wday], strtmp);
				count++;
			} else if(diffday==0) {
				printf("%s  մϴ~~~* ϨۨڨӨިͨبͨըۨڨ~ ()\n", strtmp);
				count++;
			}
		}
		
		if(birthgain=='-' || birthgain=='.') {
			btm.tm_year=varyr;
			sol2lunar(&ltm, &btm);
	
			ltm.tm_year=lunaryear;
			lunar2sol(&sltm, &ltm);
			
			sltime=mktime(&sltm);
			localtime_r(&sltime, &sltm);
			
			diffday=difftime(sltime, ntime)/60/60/24;
	
			if(diffday>0 && diffday<CLOSEBIRTH) {
				printf("%d %2d (%s)  %s (/%d%d)\n", sltm.tm_mon+1, sltm.tm_mday, weekn[sltm.tm_wday], strtmp, ltm.tm_mon+1, ltm.tm_mday);
				count++;
			} else if(diffday==0) {
				printf("%s  մϴ~~~* ϨۨڨӨިͨبͨըۨڨ~ ()\n", strtmp);
				count++;
			}
		}
	}
	
	if(!count)
		printf("%d    Ȱô±~\n", CLOSEBIRTH);
	
	fclose(fp);
}

void
printstats(char *filename, int nyear)
{
	FILE *fp;
	int count, totalage;
	char birthgain;
	char linebuffer[LONGSTR_MAX];
	char agemx_nick[MIDSTR_MAX], agemn_nick[MIDSTR_MAX];
	int yrstat[YEARMAX];
	int agemx, agemn;
	int varn, vartmp, i;
	char strtmp[MIDSTR_MAX];
	
	fp=fopen(filename, "rt");
	if(!fp) return;
	
	count=0;
	totalage=0;
	agemx=INVALID;
	agemn=INVALID;
	bzero(yrstat, YEARMAX*sizeof(*yrstat));
	
	while(!feof(fp)) {
		if(!fgets(linebuffer, LONGSTR_MAX, fp)) 
			continue;
		if(*linebuffer=='#') break;
		
		sscanf(linebuffer, "%d %d %d %c %s", &varn, &vartmp, &vartmp, &birthgain, strtmp);
		
		if(varn>100) varn-=1900;
		else if(varn<20) varn+=100;
		totalage+=varn;
		count++;
		yrstat[varn]++;
		
		if(agemx==INVALID || agemx>varn) {
			agemx=varn;
			strncpy(agemx_nick, strtmp, MIDSTR_MAX);
		}
		
		if(agemn==INVALID || agemn<=varn) {
			agemn=varn;
			strncpy(agemn_nick, strtmp, MIDSTR_MAX);
		}
	}
	
	fclose(fp);
	
	printf(" %d ڰ ϵǾ ְ,  ̴ %.1f Դϴ.\n", count, nyear-((float)totalage/count)+1);
	if(agemx!=INVALID && agemn!=INVALID)
		printf("ְڴ %s̰, ֿڴ %sԴϴ.\n", agemx_nick, agemn_nick);

	printf(" л : ");
	
	for(i=0; i<YEARMAX; i++)
		if(yrstat[i])
			printf("%d(%d) ", i, yrstat[i]);
	
	printf("\n"); 
}

void
printeldestes(char *filename)
{
	FILE *fp;
	int count;
	char linebuffer[LONGSTR_MAX];
	int varyr, varmn, varday;
	char birthgain;
	char strtmp[MIDSTR_MAX];
	
	fp=fopen(filename, "rt");
	if(!fp) return;
	
	count=0;
	
	while(!feof(fp)) {
		if(!fgets(linebuffer, LONGSTR_MAX, fp)) 
			continue;
		if(*linebuffer=='#' || count>=5) break;
		
		sscanf(linebuffer, "%d %d %d %c %s", &varyr, &varmn, &varday, &birthgain, strtmp);
		
		if(varyr>100) varyr-=1900;
		else if(varyr<20) varyr+=100;
		count++;
		
		printf("%1d. %-10s (%2d %2d)\n", count, strtmp, varyr, varmn);
	}
	
	fclose(fp);
}

void
printyoungestes(char *filename)
{
	FILE *fp;
	int count, i;
	char linebuffer[LONGSTR_MAX];
	int varyr, varmn, varday;
	char birthgain;
	char strtmp[MIDSTR_MAX];
	int listyr[RANKLIMIT], listmn[RANKLIMIT];
	char listname[RANKLIMIT][MIDSTR_MAX];
	
	fp=fopen(filename, "rt");
	if(!fp) return;
	
	count=0;
	for(i=0; i<RANKLIMIT; i++) 
		listname[i][0]=NULL;
	
	while(!feof(fp)) {
		if(!fgets(linebuffer, LONGSTR_MAX, fp)) 
			continue;
		if(*linebuffer=='#') break;
		
		sscanf(linebuffer, "%d %d %d %c %s", &varyr, &varmn, &varday, &birthgain, strtmp);
		
		if(varyr>100) varyr-=1900;
		else if(varyr<20) varyr+=100;
		count++;
		
		for(i=1; i<RANKLIMIT; i++) {
			listyr[i-1]=listyr[i];
			listmn[i-1]=listmn[i];
			strcpy(listname[i-1], listname[i]);
		}
		
		listyr[RANKLIMIT-1]=varyr;
		listmn[RANKLIMIT-1]=varmn;
		strcpy(listname[RANKLIMIT-1], strtmp);
	}
	
	fclose(fp);
	
	for(i=RANKLIMIT-1; listname[i][0] && i>=0; i--)
		printf("%1d. %-10s (%2d %2d)\n", count--, listname[i], listyr[i], listmn[i]);
}

int
main(int argc, char *argv[])
{
	time_t ntime, btime, atime;
	struct tm ntm, btm, atm;
	int year, cyear, i;
	
	time(&ntime);
	localtime_r(&ntime, &ntm);
	ntm.tm_sec=0;
	ntm.tm_min=0;
	ntm.tm_hour=0;
	ntime=mktime(&ntm);
	localtime_r(&ntime, &ntm);

	if(argc>1) {
		switch(*argv[1]) {
		case 'M':
			printf("%s\n", argv[1]+1);
			return 0;
		case 'S':
			printstats(argv[1]+1, ntm.tm_year);
			return 0;
		case 'W':
			printagegrp(argv[1]+1);
			return 0;
		case 'O':
			printeldestes(argv[1]+1);
			return 0;
		case 'Y':
			printyoungestes(argv[1]+1);
			return 0;
		case 'B':
			printclosebirth(argv[1]+1);
			return 0;
		case 'L':
			printleftbirth(argv[1]+1);
			return 0;
		default:
			year=atoi(argv[1]);
			if(year>100) year-=1900;
			else if(year<20) year+=100;
	
			if(argc<=2) 
				printf("ѱ   : %d  (%s)\n", ntm.tm_year-year+1, twelves[year%12]);
			break;
		}
	} else {
		fprintf(stderr, "Usage: %s  [ ]\n", argv[0]);
		return -1;
	}

	if(argc>2) {

		memcpy(&btm, &ntm, sizeof(struct tm));
		
		btm.tm_sec=0;
		btm.tm_min=0;
		btm.tm_hour=0;
		btm.tm_mday=atoi(argv[3]);
		btm.tm_mon=atoi(argv[2])-1;
		btm.tm_year=year;
		
		btime=mktime(&btm);
		localtime_r(&btime, &btm);

		for(i=0; zodiac[i].name; i++) {
			if(ISIN(zodiac[i].min_day, PSEUDO_YDAY(btm.tm_mon+1, btm.tm_mday), zodiac[i].max_day)) {
				if(!sol2lunar(&atm, &btm))
					printf("ѱ   : %d  (%s, %sڸ)\n", ntm.tm_year-year+1, twelves[atm.tm_year%12], zodiac[i].name);
				else 
					printf("ѱ   : %d  (%s)\n", ntm.tm_year-year+1, twelves[year%12]);
			}
		}
		
		if(btm.tm_yday>ntm.tm_yday) cyear=ntm.tm_year-btm.tm_year-1;
		else cyear=ntm.tm_year-btm.tm_year;
		
		memcpy(&atm, &btm, sizeof(struct tm));
		
		atm.tm_year+=cyear;
		
		atime=mktime(&atm);
		localtime_r(&atime, &atm);
		
		if(argc>4 && argv[4][0]=='H')
			printf("ǥ : %d %d (%d, %xh)\n", cyear, (int)(difftime(ntime, atime)/60/60/24/30.5), (int)(difftime(ntime, btime)/60/60/24), (int)(difftime(ntime, btime)/60/60/24));
		else if(argc>4 && argv[4][0]=='B') {
			char trstr[256];
			if(trans_radix((int)(difftime(ntime, btime)/60/60/24), trstr, 256, 2)) exit(1);
			printf("ǥ : %d %d (%d, %sb)\n", cyear, (int)(difftime(ntime, atime)/60/60/24/30.5), (int)(difftime(ntime, btime)/60/60/24), trstr);
		} else
			printf("ǥ : %d %d (%d)\n", cyear, (int)(difftime(ntime, atime)/60/60/24/30.5), (int)(difftime(ntime, btime)/60/60/24));
	}
	
	return 0;
}
