/*
 *  IP Whereis 1.0
 *
 *  coded by perky.
 *  Copyright(C) HanIRC FreeBSD Project Group, 2000 (except whois code)
 *
 *  This program was based on whois.c 8.1 (U.C.Berkeley, 6/6/1993)
 *
 *  @ This product includes software developed by the University of
 *    California, Berkeley and its contributors.
 *
 *  @ This program distributed under Edited BSD Licence Style.
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <err.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sysexits.h>
#include <unistd.h>

typedef struct t_whoisserver {
	char *code;
	char *dns;
	char *orghead;
	char *isphead;
} t_serv;

t_serv whoisroot = 
	{ "ARIN",	"whois.arin.net",	"",				"" };

t_serv whoisserv[] = {
	{ "RIPE",	"whois.ripe.net",	"descr:       ", "" },
	{ "APNIC",	"whois.apnic.net", 	"descr:       ", "" },
	{ "RIPN",	"whois.ripn.net",	"descr:       ", "" },
	{ NULL,		NULL,				NULL,			 NULL}
};

t_serv whoislocal[] = {
	{ "KRNIC",  "whois.nic.or.kr",  "             : ", " ISP         : " },
	{ "KRNIC",	"whois.nic.or.kr",	"         : ",    " ISP     : " },
	{ "KRNIC",	"whois.nic.or.kr",	"1c. ѱۼ񽺸  : ", " ISP     : " },
	{ NULL,		NULL,				NULL			}
};
	
#define	WHOIS_PORT			43
#define STRBUFFER_SIZE		256

#define WHOIS_RECURSE		0x01
#define WHOIS_INIC_FALLBACK	0x02
#define WHOIS_QUICK			0x04

#ifndef TRUE
#define TRUE				0x01
#endif
#ifndef FALSE
#define FALSE				0x00
#endif

static void usage __P((void));
static void whois __P((char *, struct addrinfo *, int));
static int  envfilter __P((char *));
static void doquery __P((char *, char *));

char *current_orgheader, *current_ispheader;
char orgname[STRBUFFER_SIZE], ispname[STRBUFFER_SIZE];

int
main(argc, argv)
	int argc;
	char **argv;
{
	int ch, i;
	int use_local;
	struct hostent	*hptr;
	char **pptr;
	char host[STRBUFFER_SIZE];
	char str[INET6_ADDRSTRLEN];

	use_local = 1;
	while ((ch = getopt(argc, argv, "gl")) != -1) {
		switch((char)ch) {
		case 'g':
			use_local = 0;
			break;
		case 'l':
			use_local = 1;
			break;
		case '?':
		default:
			usage();
			break;
		}
	}
	argc -= optind;
	argv += optind;

	if (!argc) {
		usage();
	}
	
	while (argc--) {
		*orgname = NULL;
		*ispname = NULL;

		if ( (hptr = (struct hostent *)gethostbyname(*argv)) == NULL) {
			fprintf(stderr, "%s : can't find address\n", *argv);
			continue;
		}
	
		switch (hptr->h_addrtype) {
		case AF_INET:
			pptr = hptr->h_addr_list;
			for ( ; *pptr != NULL; pptr++)
				strncpy(host, inet_ntop(hptr->h_addrtype, *pptr, str, sizeof(str)), STRBUFFER_SIZE);
			break;
		default:
			fprintf(stderr, "unknown address type\n");
			continue;
		}
	
		if (use_local) {
			for (i = 0; whoislocal[i].dns; i++) {
				current_orgheader = whoislocal[i].orghead;
				current_ispheader = whoislocal[i].isphead;
				doquery(whoislocal[i].dns, host);
				
				if(*orgname) break;
			}
		}
		
		if (!*orgname) {
			current_orgheader = whoisroot.orghead;
			current_ispheader = whoisroot.isphead;
			doquery(whoisroot.dns, host);
			
			if (*orgname) {
				for (i = 0; whoisserv[i].dns; i++)
					if (strstr(orgname, whoisserv[i].code)) {
						current_orgheader = whoisserv[i].orghead;
						current_ispheader = whoisserv[i].isphead;
						doquery(whoisserv[i].dns, host);
					}
			}
		}
		
		if (*orgname) {
			if(!strcmp(host,*argv))
				printf("%s : ", host);
			else 
				printf("%s(%s) : ", *argv, host);
				
			if(*ispname)
				printf("%s (%s)\n", orgname, ispname);
			else
				printf("%s\n", orgname);
		} else
			puts("Not Found");
		
		argv++;
	}
	
	exit(0);
}

static void
doquery(server, addr)
	char *server;
	char *addr;
{
	int flags, error;
	struct addrinfo hints, *res;

	memset(&hints, 0, sizeof(hints));
	hints.ai_flags = 0;
	hints.ai_family = AF_UNSPEC;
	hints.ai_socktype = SOCK_STREAM;
	error = getaddrinfo(server, "whois", &hints, &res);
	if (error != 0)
		errx(EX_NOHOST, "%s: %s", server, gai_strerror(error));

	whois(addr, res, flags);
	freeaddrinfo(res);
}

static int
envfilter(buf)
	char *buf;
{
	if (!*current_orgheader) {
		char *cutedge;
		
		strncpy(orgname, buf, STRBUFFER_SIZE);

		cutedge=strchr(orgname, '(');
		if(cutedge) 
			cutedge[-1]=NULL;

		return TRUE;
	} else if (!strncmp(current_orgheader, buf, strlen(current_orgheader))) {
		strncpy(orgname, buf + strlen(current_orgheader), STRBUFFER_SIZE);
		return TRUE;
	} else if (*current_ispheader) {
		if (!strncmp(current_ispheader, buf, strlen(current_ispheader)))
			strncpy(ispname, buf + strlen(current_ispheader), STRBUFFER_SIZE);
		return FALSE;
	} else 
		return FALSE;
}

static void
whois(name, res, flags)
	char *name;
	struct addrinfo *res;
	int flags;
{
	FILE *sfi, *sfo;
	char *buf, *p, *nhost;
	size_t len;
	int s, nomatch;

	s = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
	if (s < 0) {
		err(EX_OSERR, "socket");
	}

	if (connect(s, res->ai_addr, res->ai_addrlen) < 0) {
		err(EX_OSERR, "connect");
	}

	sfi = fdopen(s, "r");
	sfo = fdopen(s, "w");
	if (sfi == NULL || sfo == NULL) {
		err(EX_OSERR, "fdopen");
	}
	(void)fprintf(sfo, "%s\r\n", name);
	(void)fflush(sfo);
	nhost = NULL;
	nomatch = 0;
	while ((buf = fgetln(sfi, &len))) {
		if (buf[len - 2] == '\r') {
			buf[len - 2] = '\0';
		} else {
			buf[len - 1] = '\0';
		}

		if ((flags & WHOIS_RECURSE) && !nhost &&
		    (p = strstr(buf, "Whois Server: "))) {
			p += sizeof("Whois Server: ") - 1;
			if ((len = strcspn(p, " \t\n\r"))) {
				if ((nhost = malloc(len + 1)) == NULL) {
					err(1, "malloc");
				}
				memcpy(nhost, p, len);
				nhost[len] = '\0';
			}
		}
		if ((flags & WHOIS_INIC_FALLBACK) && !nhost && !nomatch &&
		    (p = strstr(buf, "No match for \""))) {
			p += sizeof("No match for \"") - 1;
			if ((len = strcspn(p, "\"")) &&
			    strncasecmp(name, p, len) == 0 &&
			    name[len] == '\0' &&
			    strchr(name, '.') == NULL) {
				nomatch = 1;
			}
		}
		if (envfilter(buf)) 
			break;
	}

	if (nhost) {
		struct addrinfo hints, *res2;
		int error;

		memset(&hints, 0, sizeof(hints));
		hints.ai_flags = 0;
		hints.ai_family = AF_UNSPEC;
		hints.ai_socktype = SOCK_STREAM;
		error = getaddrinfo(nhost, "whois", &hints, &res2);
		if (error != 0) {
			warnx("%s: %s", nhost, gai_strerror(error));
			return;
		}
		if (!nomatch) {
			free(nhost);
		}
		whois(name, res2, 0);
		freeaddrinfo(res2);
	}
}

static void
usage()
{
	(void)fprintf(stderr,
	    "usage: ipwhereis [-k] {ipv4 address}|{dns name} ...\n");
	exit(EX_USAGE);
}
