#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ctype.h>
#include <sys/stat.h>
#include <sys/types.h>

#define STR_COLORMAX		20
#define STR_DIRECTIONMAX	10
#define STR_COMMENTMAX		80

struct luckinfo {
	char luckcolor[STR_COLORMAX];
	int  lucknum;
	char luckdir[STR_DIRECTIONMAX];
	char comment[STR_COMMENTMAX];
};

#define LINEBUF_MAX		1024
#define TRUE			1
#define FALSE			0
#define DOUBLE_INVALID	-290.0
#define ISVALID(n)		((n)>DOUBLE_INVALID)

#define DATAHEADLINE	"<IMG SRC=\"/online_news/image/luck"

void deltags(char *str);
void strsubst(char *str, const char *org, const char *dst);
int  dequote_str(char *str, char *quotes);
int  getinfo(FILE *fp, struct luckinfo *ldata, int year);

char *ttiname[12]={	"", "", "ȣ", "䳢", "", "", "", "", "", "", "", "" };

int
main(int argc, char *argv[])
{
    FILE *fp;
    char linebuf[LINEBUF_MAX];
    struct luckinfo ldata;
    int headlen, nowhere, ttin, tryear;
    
    if(argc<3) {
    	fprintf(stderr, "not enough args\n");
    	return -1;
    }
    
    if(!strncasecmp(argv[2], "up", 2)) {	/* update time; */
    	struct stat sb;
    	
    	if(stat(argv[1], &sb)) {
    		fprintf(stderr, "file open error\n");
    		exit(0);
    	} else {
    		printf("ֱ  Ʈ : %s\n", asctime(localtime(&(sb.st_mtime))));
    		exit(0);
    	}
    }
    
    fp = fopen(argv[1], "rt");
    if(fp==NULL) {
    	fprintf(stderr, "inputfile open error\n");
    	return -1;
    }
   
    headlen = strlen(DATAHEADLINE);
	nowhere = TRUE;
    tryear  = atoi(argv[2]);
	if(!isdigit(*argv[2]) || tryear==0) {
		printf(" ̰ ƴϿ. ̳ ̸ ּ.\n");
		exit(0);
	}

	if(tryear>100) tryear-=1900;
	else if(tryear<20) tryear+=100;
    
	while(!feof(fp)) {
		if(!fgets(linebuf, LINEBUF_MAX, fp)) continue;
		
		if(!strncmp(DATAHEADLINE, linebuf, headlen)) {
			ttin=atoi(linebuf+headlen);
			if((ttin-1)!=tryear%12) continue;
			
			printf("[%s] ", ttiname[ttin-1]);
			
			if(!getinfo(fp, &ldata, tryear)) {
				nowhere=FALSE;
				printf("  <%s> /   <%d> /   <%s>\n", ldata.luckcolor, 
																		ldata.lucknum, ldata.luckdir);
				if(*(ldata.comment)) 
					printf("[%s]   : %s\n", ttiname[ttin-1], ldata.comment);
			}
		}
	}
	
	if(nowhere)
		printf("%s :   ϴ.\n", argv[2]);
	
	fclose(fp);
    
    return NULL;
}

int
getinfo(FILE *fp, struct luckinfo *ldata, int year)
{
	int i;
	char linebuf[LINEBUF_MAX], commentbuff[LINEBUF_MAX];
	
	*commentbuff=NULL;
	*(ldata->comment)=NULL;
	for(i=0; i<5 && !feof(fp);) {
		if(!fgets(linebuf, LINEBUF_MAX, fp))
			continue;
		
		deltags(linebuf);
		dequote_str(linebuf, " \t\n\r");
		if(!*linebuf) continue;
		
		switch(i) {
		case 0: /* : ȸ	*/
				strncpy(ldata->luckcolor, linebuf+strlen(" : "), STR_COLORMAX);
				break;
		case 1: /* : 8*/
				ldata->lucknum = atoi(linebuf+strlen(" : "));
				break;
		case 2: /* : */
				strncpy(ldata->luckdir, linebuf+strlen(" : "), STR_DIRECTIONMAX);
				break;
		case 3: /*24  ϶.36 ð Ʋ.48   ´.60 Ӹ Ų*/
				strncpy(commentbuff, linebuf, LINEBUF_MAX);
				break;
		case 4: /* ִ.72    .*/
				strcat(commentbuff, linebuf);
				break;
		}
		
		i++;
	}
	
	if(*commentbuff && year) {
		char yearpatt[STR_COMMENTMAX], *sptr;
		
		snprintf(yearpatt, STR_COMMENTMAX, "%d", year);
		
		sptr=strstr(commentbuff, yearpatt);
		if(sptr) {
			char *str;
			
			sptr+=strlen(yearpatt);
			str=sptr;
			
			while(*str && *str!='\n' && *str!='\r' && !isdigit(*str)) {
				if(*(str++)=='.') {
					*str=NULL;
					break;
				}
			}
			*str=NULL;
			strncpy(ldata->comment, sptr, STR_COMMENTMAX);
			dequote_str(ldata->comment, " \t\n\r");
		}
	}
	
	return NULL;
}

void
deltags(char *str)
{
	int intag;
	char *cbuf, *cbufcur, *strcur;
	
	cbufcur=cbuf=(char*)malloc(strlen(str));
	
	intag=NULL;
	
	for(strcur=str; *strcur; strcur++) {
		if(*strcur=='<')
			intag=TRUE;
		else if(*strcur=='>' && intag)
			intag=FALSE;
		else if(!intag && *strcur!='\t' && *strcur!='\n' && *strcur!='\r') {
			*(cbufcur++)=*strcur;
		}
	}
	
	*cbufcur=NULL;
	
	strcpy(str, cbuf);
	
	free(cbuf);
}

void
strsubst(char *str, const char *org, const char *dst)
{
	int seps=strlen(org), i;
	
	while(*str) {
		for(i=0; i<seps; i++)
			if(org[i]==*str)
				*str=dst[i];
		str++;
	}
}

int
dequote_str(char *str, char *quotes)
{
	int i;
	char *cur;
	int orglen=strlen(str), nquote=strlen(quotes);
	
	for(cur=str+orglen-1; cur>=str; cur--) {	/* delete backing quotes */
		for(i=0; i<nquote; i++) 
			if(*cur==quotes[i]) {
				*cur=NULL;
				break;
			}
		if(i==nquote) break;
	}
	
	for(cur=str; *cur; cur++) {
		for(i=0; i<nquote; i++)
			if(*cur==quotes[i]) 
				break;
		if(i==nquote) break;
	}
	
	if(cur>str)
		strcpy(str, cur);

	return TRUE;
}
