#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <sys/stat.h>
#include <sys/types.h>

struct kmainfo {
	char location[21];
	char weather[51];
	double celsius;
	char winddir[9];
	double windvel;
	double humidity;
	double rain, snow;
};

#define LINEBUF_MAX		1024
#define TRUE			1
#define FALSE			0
#define DOUBLE_INVALID	-290.0
#define ISVALID(n)		((n)>DOUBLE_INVALID)
#define C2F(cel)		(9.0/5.0*(cel)+32.0)
#define F2C(fah)		((fah)-32.0)*5.0/9.0)

#define DATAHEADLINE	"<TR bgcolor=#e2ecdd>"

struct limdata {
	double limit;
	char *name;
};

struct limdata windchillindex[]={
	{	  75.0,	""			},
	{	 150.0,	""		},
	{	 300.0, ""		},
	{	 500.0, ""		},
	{	 700.0, "ſ "	},
	{	 900.0, "߿"			},
	{	1100.0, "ſ ߿"		},
	{	1300.0, "ص ߿"	},
	{	1650.0, " Ե"	},
	{	2100.0, "1г  " },
	{DOUBLE_INVALID, "30ʳ  " }
};

struct limdata discomfortindex[]={
	{	  68.0, " "		},
	{	  70.0, "踦 Ÿ" },
	{	  75.0, "10% "  },
	{	  80.0, "50% "  },
	{	  83.0, " "		},
	{DOUBLE_INVALID, "ſ " }
};

void deltags(char *str);
void strsubst(char *str, const char *org, const char *dst);
int  dequote_str(char *str, char *quotes);
void prinfo(char *location, char *str);
void translate_direction(char *str);
int  getwinfo(FILE *fp, struct kmainfo *wdata);


int
main(int argc, char *argv[])
{
    FILE *fp;
    char linebuf[LINEBUF_MAX];
    struct kmainfo wdata;
    double tmp;
    int headlen, i, nowhere;
    
    if(argc<3) {
    	fprintf(stderr, "not enough args\n");
    	return -1;
    }
    
    if(!strncasecmp(argv[2], "up", 2)) {	/* update time; */
    	struct stat sb;
    	
    	if(stat(argv[1], &sb)) {
    		fprintf(stderr, "file open error\n");
    		exit(0);
    	} else {
    		printf("ֱ  Ʈ : %s\n", asctime(localtime(&(sb.st_mtime))));
    		exit(0);
    	}
    }
    
    fp = fopen(argv[1], "rt");
    if(fp==NULL) {
    	fprintf(stderr, "inputfile open error\n");
    	return -1;
    }
   
    headlen = strlen(DATAHEADLINE);
	nowhere = TRUE;
    
	while(!feof(fp)) {
		if(!fgets(linebuf, LINEBUF_MAX, fp)) continue;
		
		if(!strncmp(DATAHEADLINE, linebuf, headlen)) {
			if(!getwinfo(fp, &wdata) && !strcmp(argv[2], wdata.location)) {
				nowhere=FALSE;
				if(wdata.weather[0]) {
					snprintf(linebuf, LINEBUF_MAX, " : %s", wdata.weather);
					prinfo(wdata.location, linebuf);
				}
				if(ISVALID(wdata.celsius)) {
					snprintf(linebuf, LINEBUF_MAX, "µ : %.1f ", wdata.celsius);
					prinfo(wdata.location, linebuf);
				}
				if(ISVALID(wdata.humidity)) {
					snprintf(linebuf, LINEBUF_MAX, " : %.1f ", wdata.humidity);
					prinfo(wdata.location, linebuf);
				}
				if(ISVALID(wdata.windvel)) {
					if(wdata.windvel==0.0)
						snprintf(linebuf, LINEBUF_MAX, "ٶ : ");
					else
						snprintf(linebuf, LINEBUF_MAX, "ٶ : %.1f  (%s)", wdata.windvel, wdata.winddir);
					prinfo(wdata.location, linebuf);
				}
				if(ISVALID(wdata.rain)) {
					snprintf(linebuf, LINEBUF_MAX, " : %.1f ", wdata.rain);
					prinfo(wdata.location, linebuf);
				}
				if(ISVALID(wdata.snow)) {
					snprintf(linebuf, LINEBUF_MAX, " : %.1f ", wdata.snow);
					prinfo(wdata.location, linebuf);
				}
				if(ISVALID(wdata.celsius) && ISVALID(wdata.windvel)) {
					snprintf(linebuf, LINEBUF_MAX, "üµ : %.1f ", 33 - ((10.45 + 10 * sqrt(wdata.windvel) - wdata.windvel)*(33 - wdata.celsius)) / 22.04);
					prinfo(wdata.location, linebuf);
				}
				if(ISVALID(wdata.celsius) && ISVALID(wdata.humidity)) {
					tmp=(C2F(wdata.celsius) - 0.55*(1.0-wdata.humidity/100.0)*(C2F(wdata.celsius)-58));
					for(i=0; discomfortindex[i].limit>DOUBLE_INVALID && discomfortindex[i].limit<tmp; i++) ;
					snprintf(linebuf, LINEBUF_MAX, " : %d (%s)", (int)tmp, discomfortindex[i].name);
					prinfo(wdata.location, linebuf);
				}
				if(ISVALID(wdata.windvel) && ISVALID(wdata.celsius)) {
					tmp=(sqrt(100.0*wdata.windvel) - wdata.windvel + 10.5) * (33 - wdata.celsius);
					for(i=0; windchillindex[i].limit>DOUBLE_INVALID && windchillindex[i].limit<tmp; i++) ;
					snprintf(linebuf, LINEBUF_MAX, "ð : %d (%s)", (int)tmp, windchillindex[i].name);
					prinfo(wdata.location, linebuf);
				}
				
				prinfo(wdata.location, NULL);
			}
		}
	}
	
	if(nowhere)
		printf("%s : ѱ û   Դϴ.\n", argv[2]);
	
	fclose(fp);
    
    return NULL;
}

#define INFO_MAX	3

void prinfo(char *location, char *str)
{
	static int ninfo=NULL;
	static char infobuff[INFO_MAX][LINEBUF_MAX];
	int i;
	
	if(str) {
		strncpy(infobuff[ninfo], str, LINEBUF_MAX);
		ninfo++;
	}

	if((!str || ninfo>=INFO_MAX) && ninfo) {
		printf("[%s] ", location);
		
		for(i=0; i<ninfo; i++) {
			if(i) 
				printf(" / ");
			printf("%s", infobuff[i]);
		}
		printf("\n");
		ninfo=NULL;
	}
}

int
getwinfo(FILE *fp, struct kmainfo *wdata)
{
	int i;
	char linebuf[LINEBUF_MAX];
	
	for(i=0; i<8 && !feof(fp);) {
		if(!fgets(linebuf, LINEBUF_MAX, fp))
			continue;
		
		deltags(linebuf);
		
		switch(i) {
		case 0:	strcpy(wdata->location, linebuf);		break;
		case 1:	strcpy(wdata->weather, linebuf);		break;
		case 2:	if(*linebuf!=NULL)
					wdata->celsius=atof(linebuf);
				else wdata->celsius=DOUBLE_INVALID;		break;
		case 3:	translate_direction(linebuf);
				strcpy(wdata->winddir, linebuf);		break;
		case 4:	if(*linebuf!=NULL)
					wdata->windvel=atof(linebuf);
				else wdata->windvel=DOUBLE_INVALID;		break;
		case 5:	if(*linebuf!=NULL)
					wdata->humidity=atof(linebuf);
				else wdata->humidity=DOUBLE_INVALID;	break;
		case 6:	if(*linebuf==NULL)
					wdata->rain=DOUBLE_INVALID;
				else wdata->rain=atof(linebuf);			break;
		case 7:	if(*linebuf==NULL)
					wdata->snow=DOUBLE_INVALID;
				else wdata->snow=atof(linebuf);			break;
		}
		
		i++;
	}
	
	return NULL;
}

void
translate_direction(char *str)
{
	char *cbuf, *strcur, tbuff[2]={NULL, NULL};
	
	cbuf=(char*)malloc(strlen(str));
	
	*cbuf=NULL;
	
	for(strcur=str; *strcur; strcur++) {
		switch(*strcur) {
		case 'N':
			strcat(cbuf, "");
			break;
		case 'E':
			strcat(cbuf, "");
			break;
		case 'W':
			strcat(cbuf, "");
			break;
		case 'S':
			strcat(cbuf, "");
			break;
		default:
			*tbuff=*strcur;
			strcat(cbuf, tbuff);
			break;
		}
	}
	
	strcpy(str, cbuf);
	
	free(cbuf);
}

void
deltags(char *str)
{
	int intag;
	char *cbuf, *cbufcur, *strcur;
	
	cbufcur=cbuf=(char*)malloc(strlen(str));
	
	intag=NULL;
	
	for(strcur=str; *strcur; strcur++) {
		if(*strcur=='<')
			intag=TRUE;
		else if(*strcur=='>' && intag)
			intag=FALSE;
		else if(!intag && *strcur!=' ' && *strcur!='\t' && *strcur!='\n' && *strcur!='\r') {
			*(cbufcur++)=*strcur;
		}
	}
	
	*cbufcur=NULL;
	
	strcpy(str, cbuf);
	
	free(cbuf);
}

void
strsubst(char *str, const char *org, const char *dst)
{
	int seps=strlen(org), i;
	
	while(*str) {
		for(i=0; i<seps; i++)
			if(org[i]==*str)
				*str=dst[i];
		str++;
	}
}

int
dequote_str(char *str, char *quotes)
{
	int i;
	char *cur;
	int orglen=strlen(str), nquote=strlen(quotes);
	
	for(cur=str+orglen-1; cur>=str; cur--) {	/* delete backing quotes */
		for(i=0; i<nquote; i++) 
			if(*cur==quotes[i]) {
				*cur=NULL;
				break;
			}
		if(i==nquote) break;
	}
	
	for(cur=str; *cur; cur++) {
		for(i=0; i<nquote; i++)
			if(*cur==quotes[i]) 
				break;
		if(i==nquote) break;
	}
	
	if(cur>str)
		strcpy(str, cur);

	return TRUE;
}
