#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#define ERR_TRANS_RADIX_INVALID		-1
#define ERR_TRANS_RADIX_OK			 0

int
trans_radix(int value, char *str, int len, int radixdst)
{
	register char *trc;
	register int  digit;
	
	for(trc=str+len-2, *(trc+1)=NULL; trc>=str; --trc) {
		digit=value%radixdst;
		if(digit<10)
			*trc=digit+'0';
		else
			*trc=digit-10+'A';
		if(!(value/=radixdst)) break;
	}
	
	if(trc<str)
		return ERR_TRANS_RADIX_INVALID;
	else if(trc==str)
		return ERR_TRANS_RADIX_OK;
	else {
		while(*trc) 
			*(str++)=*(trc++);
		*str=NULL;
		return ERR_TRANS_RADIX_OK;
	}
	
	#undef TOASCII
}

int
decode_radix(char *str, int radixsrc)
{
	register char *trc;
	register int  mulx;
	int result;
	
	mulx=1;
	result=0;
	
	for(trc=str+strlen(str)-1; trc>=str; trc--) {
		if(isdigit(*trc)) {
			result+=mulx*(*trc-'0');
			mulx*=radixsrc;
		} else if(islower(*trc)) {
			result+=mulx*(*trc-'a'+10);
			mulx*=radixsrc;
		} else if(isupper(*trc)) {
			result+=mulx*(*trc-'A'+10);
			mulx*=radixsrc;
		}
	}
	
	return result;
}

void
usage(char *cmd)
{
	fprintf(stdout, "Usage: %s {number to convert} {source radix} [destinatioin radix]\n", cmd);
}

int
main(int argc, char *argv[])
{
	#define RDXSTR_LEN	256
	char rdxstr[RDXSTR_LEN];
	int srcradix, dstradix;
	
	if(argc<3) {
		usage(argv[0]);
		return 0;
	}
	
	srcradix=atoi(argv[2]);
	if(argc>=4)
		dstradix=atoi(argv[3]);
	else
		dstradix=10;
		
	if(srcradix<0 || srcradix>36 || dstradix<0 || dstradix>36) {
		printf("Doesn't support that radix number.\n");
		return 0;
	}
	
	if(trans_radix(decode_radix(argv[1], atoi(argv[2])), rdxstr, RDXSTR_LEN, (argc>=4) ? atoi(argv[3]) : 10))
		fprintf(stderr, "Error in converting\n");
	else
		fprintf(stdout, "%s\n", rdxstr);
	
	return NULL;
}
