#
# @(#)Chosun/Plugin.pm
#
# This gets the latest Chosun.com news
#
# Author      : Kilsang Kang <biko@kang.sarang.net>
# Date        : 2001/10/08 ()
# Revision    : $Revision: 1.1.1.1 $
# Last Update : $Date: 2002/09/03 09:15:12 $
#

package Chosun::Plugin;

use Perlbot;
use POSIX;
use HTML::TokeParser;
use HTTP::Lite;

sub get_hooks {
  return { public => \&on_public, msg => \&on_msg };
}

sub on_public {
  my $conn = shift;
  my $event = shift;
  my $args;

  ($args = $event->{args}[0]) =~ tr/[A-Z]/[a-z]/;

  if($args =~ /^-(chosun|)\s*/) {
    get_chosun($conn, $event, $event->{to}[0]);
  }
}

sub on_msg {
  my $conn = shift;
  my $event = shift;
  my $args;

  ($args = $event->{args}[0]) =~ tr/[A-Z]/[a-z]/;

  if($args =~ /^-(chosun|)\s*/) {
    get_chosun($conn, $event, $event->nick);
  }
}

sub get_chosun {
  my $conn  = shift;
  my $event = shift;
  my $who   = shift;
  my $args  = $event->{args}[0];
  my $max   = "";

  $args =~ tr/[A-Z]/[a-z]/;
  $args =~ s/^-(chosun|)\s*//;

  my @argv = split(/ /, $args);
  my %SECTION = (
  'p__' => ['ġ', 'politics'],
  'e__' => ['', 'economy'],
  'o__' => ['ȸ', 'national'],
  'w__' => ['', 'international'],
  'm__' => ['ȭ', 'culture'],
  's__' => ['', 'sports'],
  'i__' => ['', 'it'],
  'BAL' => ['', 'total']
  );
  my $section = '';
  my $limit = 5;

  if ($#argv > -1) {
    foreach my $key (keys %SECTION) {
      foreach (@{$SECTION{$key}}) {
        if (/^$argv[0]/) { $section = $key; last; }
      }
      last if ($section);
    }
  }
  unless ($section) { $section = 'BAL'; }
  $limit = int($argv[$#argv]) if($argv[$#argv] =~ /^(\d+)$/);
  $limit = 5 if ($limit <= 0);
  $limit =20 if ($limit > 20);

  if(!defined($pid = fork)) {
    $conn->privmsg($who, "error in chosun.com news plugin...");
    return;
  }

  if($pid) {
    #parent
  
    $SIG{CHLD} = sub { wait; };
    return;
  
  } else {
    # child

    $conn->privmsg($who, "Ϻ(http://www.chosun.com/)");

    my $stat = 0;
    my $http = new HTTP::Lite;
	my $url = "http://www.chosun.com/w21data/html/list/${section}.html";
  
    if (($stat = $http->request($url)) != 200) {
       $conn->privmsg($who, "server error: $stat");
       $conn->{_connected} = 0;
       exit 0;
    }

    my $p = HTML::TokeParser->new(\$http->body) || die "Can't open: $!\n";
	my $text = undef;
	my $photo = undef;

    ### search <a href="$url">
    while(my $token = $p->get_tag("a")) {
      if ($token->[1]{href} =~ m#/w21data/html/list/${section}.html#) {
        last;
      }
    }

    while(my $token = $p->get_tag("a")) {
      if ($token->[1]{href} =~ m#/w21data/html/news/(\d+)/(\d+)\.html#) {
        next unless($text = $p->get_trimmed_text("/a"));

        $photo = 0;
        while($token = $p->get_token) {
          if ($token->[0] eq 'S' and $token->[1]=~/img/i) {
    	    $photo = 1; last;
          }
          elsif ($token->[0] eq 'E' && $token->[1]=~/td/i) {
            last;
          }
        }

        $text =~ s/^\s*//;
        $text =~ s/\s*$//;

        next unless ($text);

        my $send= '['.$SECTION{$section}[0].'] '.$text.($photo?' [PHOTO]':''); 
        $conn->privmsg($who, "$send");
        last unless(--$limit);
      }
    }

    $conn->{_connected} = 0;
    exit 0;
  }
}

1;
