# Joh, Yong-iL <tolkien@nownuri.net>
#
# This gets the latest DaemonNews entries.

package DaemonNews::Plugin;

use Perlbot;

use Socket;
use POSIX;
require HTML::TokeParser;

sub get_hooks {
  return { public => \&on_public, msg => \&on_msg };
}

sub on_public {
  my $conn = shift;
  my $event = shift;
  my $args;

  ($args = $event->{args}[0]) =~ tr/[A-Z]/[a-z]/;

  if($args =~ /^-daemonnews\s*/  || $args =~ /^-dn\s*/) {
      get_dn($conn, $event, $event->{to}[0]);
  }
}

sub on_msg {
  my $conn = shift;
  my $event = shift;
  my $args;

  ($args = $event->{args}[0]) =~ tr/[A-Z]/[a-z]/;

  if($args =~ /^-daemonnews\s*/  || $args =~ /^-dn\s*/) {
      get_dn($conn, $event, $event->nick);
  }
}

sub get_dn {
  my $conn = shift;
  my $event = shift;
  my $who = shift;

  $args = $event->{args}[0];
  my($term, $min, $max) = split(/ /, $args, 3);

  $term =~ s/^-daemonnews\s*//;
  $term =~ s/^-dn\s*//;
  $term =~ s/\s+/+/g;

  $min =~ s/.*(\d+).*/\1/;
  $max =~ s/.*(\d+).*/\1/;

  if ($max eq '' || $max < 1) { $max = 5; }
  if ($min eq '' || $min < 1) { $min = 0;
  } else { $min -= 1; }

  if(!defined($pid = fork)) {
    $conn->privmsg($who, "error in DaemonNews plugin...");
    return;
  }

  if($pid) {
    #parent

    $SIG{CHLD} = sub { wait; };
    return;

  } else {
    # child

    my($remote,$port,$iaddr,$paddr,$proto,$line);
    $remote = "daily.daemonnews.org";
    $port = "80";

    if(!defined($iaddr = inet_aton($remote))) {
      $conn->privmsg($who, "Could not get address for $remote");
      $conn->{_connected} = 0;
      exit 1;
    }
    if(!defined($paddr = sockaddr_in($port, $iaddr))) {
      $conn->privmsg($who, "Could not get port for $remote");
      $conn->{_connected} = 0;
      exit 1;
    }
    if(!defined($proto = getprotobyname('tcp'))) {
      $conn->privmsg($who, "Could not get tcp protocol");
      $conn->{_connected} = 0;
      exit 1;
    }

    if(!socket(SOCK, PF_INET, SOCK_STREAM, $proto)) {
      $conn->privmsg($who, "Could not open socket to $remote");
      $conn->{_connected} = 0;
      exit 1;
    }
    if(!connect(SOCK, $paddr)) {
      $conn->privmsg($who, "Could not connect to $remote");
      $conn->{_connected} = 0;
      exit 1;
    }

    $msg = "GET / HTTP/1.1\nHost: daily.daemonnews.org\nUser-Agent: Lynx\n\n";

    if(!send(SOCK, $msg, 0)) {
      $conn->privmsg($who, "Could not send to $remote");
      $conn->{_connected} = 0;
      exit 1;
    }
    my $p = HTML::TokeParser->new(*SOCK) || die "Can't open: $!";

    $conn->privmsg($who, "Daemon News:");

    my (@tmp, $i, $lcnt, $dummy);
    $dummy = $min + $max - 1;

    $lcnt = 0;
    while (my $token = $p->get_token) {
      if ($token->[0] eq 'C') {
	if ($token->[1] =~ /Navigation bar/) { last; }
	if ($token->[1] =~ /START/) {
	  $lcnt++;
	  $token = $p->get_tag("table");
	  $token = $p->get_tag("table");
	  $token = $p->get_tag("font");
	  my $title = $p->get_trimmed_text("/font");
	  $token = $p->get_tag("a");
	  my $url = $token->[1]{href} || "-";
	  $token = $p->get_tag("td");
	  $token = $p->get_tag("font");
	  my $date = $p->get_trimmed_text("/font");

	  @tmp = split(/ /, $date);
	  $tmp[1] = substr($tmp[1], 0, 3);
	  $tmp[2] = substr($tmp[2], 2, 2);

	  $conn->privmsg($who, "\[$tmp[1]/$tmp[0]/$tmp[2]\] $title\n");
	  if ($url ne "-") {
	    $conn->privmsg($who, "            $url\n");
	  }
	}
      } else { next; }
      if ($lcnt > $dummy) { last; }
    }
    close SOCK;
    $conn->{_connected} = 0;
    exit 0;
  }
  close SOCK;
}

1;
