# Andrew Burke <burke@pas.rochester.edu>
#
# This gets the latest Freshmeat entries.
#
# modified by Yong-iL Joh <tolkien@nownuri.net>
# some changes in search

package Freshmeat::Plugin;

use Perlbot;

use Socket;
use POSIX;
use XML::Parser;
use XML::SimpleObject;

sub get_hooks {
  return { public => \&on_public, msg => \&on_msg };
}

sub on_public {
  my $conn = shift;
  my $event = shift;
  my $args;

  ($args = $event->{args}[0]) =~ tr/[A-Z]/[a-z]/;

  if($args =~ /^-freshmeat\s*/  || $args =~ /^-fm\s*/) {
    if($args =~ /^-freshmeat\s+search.*/ || $args =~/^-fm\s+search.*/) {
      get_fm_search($conn, $event, $event->{to}[0]);
    } else {
      get_fm($conn, $event, $event->{to}[0]);
    } 
  }
}

sub on_msg {
  my $conn = shift;
  my $event = shift;
  my $args;
 
  ($args = $event->{args}[0]) =~ tr/[A-Z]/[a-z]/;

  if($args =~ /^-freshmeat\s*/  || $args =~ /^-fm\s*/) {
    if($args =~ /^-freshmeat\s+search.*/ || $args =~/^-fm\s+search.*/) {
      get_fm_search($conn, $event, $event->nick);
    } else {
      get_fm($conn, $event, $event->nick);
    } 
  }
}

sub get_fm {
  my $conn = shift;
  my $event = shift;
  my $who = shift;
  my $max;

  ($max = $event->{args}[0]) =~ tr/[A-Z]/[a-z]/;
  $max =~ s/^-freshmeat\s*//;
  $max =~ s/^-fm\s*//;
  $max =~ s/\s+(\d+)\s*.*/\1/;

  if($max eq '') { $max = 5; }

  if(!defined($pid = fork)) {
    $conn->privmsg($who, "error in freshmeat plugin...");
    return;
  }

  if($pid) {
    #parent
    
    $SIG{CHLD} = sub { wait; };
    return;

  } else {
    # child

    open FILE, $datadir . "/freshmeat.xml" || goto EXIT_FM;

    my @line = <FILE>;
    my $i = 1;
    my $parser = new XML::Parser (ErrorContext => 2, Style => "Tree");
    my $xmlobj = new XML::SimpleObject ($parser->parse(join(' ', @line)));

    my $tmp = $xmlobj->child("rss")->child("channel");
    my $date = $tmp->child("pubDate")->value;
    my ($day, $month, $year, $time) = $date
	=~ /.*?,\s*(\d+)\s*(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\s*(\d\d\d\d)\s*(\d+:\d+):\d+.*/;
    $month =~ s/Jan/01/;
    $month =~ s/Feb/02/;
    $month =~ s/Mar/03/;
    $month =~ s/Apr/04/;
    $month =~ s/May/05/;
    $month =~ s/Jun/06/;
    $month =~ s/Jul/07/;
    $month =~ s/Aug/08/;
    $month =~ s/Sep/09/;
    $month =~ s/Oct/10/;
    $month =~ s/Nov/11/;
    $month =~ s/Dec/12/;

    $conn->privmsg($who, "Freshmeat Headlines: $year/$month/$day $time");

    foreach my $element ($tmp->children("item")) {
	my $title = $element->child("title")->value;
	my $link = $element->child("link")->value;
	my $desc = $element->child("description")->value;

	if($i > $max) { last; }
	my $short_title = substr($title, 0, 41);
	my ($short_link) = $link
		=~ /http:\/\/freshmeat.net\/releases\/(\d+)\//;
	my $output = sprintf("%42s / articleNo.%s\n", $short_title, $short_link);
	$conn->privmsg($who, $output);
	$conn->privmsg($who, " " . $desc);

	$i++;
	next;
    }

  EXIT_FM:
    close FILE;
    $conn->{_connected} = 0;
    exit 0;
  }

  close SOCK;
  
}

sub get_fm_search {
  my $conn = shift;
  my $event = shift;
  my $who = shift;
  my $args;
  my $max;

  $args = $event->{args}[0];

  my($term, $max) = split(':', $args, 2);

  $term =~ s/^-freshmeat\s+search\s*//;
  $term =~ s/^-fm\s+search\s*//;
  $term =~ s/\s+/+/g;


  $max =~ s/.*:\s*(\d+).*/\1/;

  if($max eq '' || $max < 1) { $max = 3; }

  if(!defined($pid = fork)) {
    $conn->privmsg($who, "error in freshmeat plugin...");
    return;
  }

  if($pid) {
    #parent
    
    $SIG{CHLD} = sub { wait; };
    return;

  } else {
    # child

    my($remote,$port,$iaddr,$paddr,$proto,$line);
    $remote = "freshmeat.net";
    $port = "80";
    
    if(!defined($iaddr = inet_aton($remote))) {
      $conn->privmsg($who, "Could not get address for $remote");
      $conn->{_connected} = 0;
      exit 1;
    }
    if(!defined($paddr = sockaddr_in($port, $iaddr))) {
      $conn->privmsg($who, "Could not get port for $remote");
      $conn->{_connected} = 0;
      exit 1;
    }
    if(!defined($proto = getprotobyname('tcp'))) {
      $conn->privmsg($who, "Could not get tcp protocol");
      $conn->{_connected} = 0;
      exit 1;
    }
    
    if(!socket(SOCK, PF_INET, SOCK_STREAM, $proto)) {
      $conn->privmsg($who, "Could not open socket to $remote");
      $conn->{_connected} = 0;
      exit 1;
    }
    if(!connect(SOCK, $paddr)) {
      $conn->privmsg($who, "Could not connect to $remote");
      $conn->{_connected} = 0;
      exit 1;
    }

    $msg = "GET /search-xml?q=$term\n\n";

    if(!send(SOCK, $msg, 0)) {
      $conn->privmsg($who, "Could not send to $remote");
      $conn->{_connected} = 0;
      exit 1;
    }

    my $title = '';
    my $desc = '';
    my $homepage = '';

    my @line = <SOCK>;
    my $i = 1;
    my $parser = new XML::Parser (ErrorContext => 2, Style => "Tree");
    my $xmlobj = new XML::SimpleObject ($parser->parse(join(' ', @line)));

    foreach my $element ($xmlobj->child("search-results")->children("match")) {
	$title = $element->child("projectname_full")->value;
	$desc = $element->child("desc_short")->value;
	$homepage = $element->child("url_homepage")->value;

	if($i > $max) { last; }
	$conn->privmsg($who, "$title / $desc");
	if ($homepage ne '') {
	    $conn->privmsg($who, "  $homepage");
	}
	
	$title = '';
	$desc = '';
	$homepage = '';
	
	$i++;
    }

    close SOCK;
    $conn->{_connected} = 0;
    exit 0;
  }

  close SOCK;
  
}

1;
