# Joh Yong-iL <tolkien@nownuri.net>
#
# This gets the latest Ѱܷ news

package Hani::Plugin;

use Perlbot;

use POSIX;
use HTTP::Lite;
require HTML::TokeParser;

sub get_hooks {
  return { public => \&on_public, msg => \&on_msg };
}

sub on_public {
  my $conn = shift;
  my $event = shift;
  my $args;

  ($args = $event->{args}[0]) =~ tr/[A-Z]/[a-z]/;

  if($args =~ /^-(hani|Ѱܷ)\s*/) {
    get_hani($conn, $event, $event->{to}[0]);
  }
}

sub on_msg {
  my $conn = shift;
  my $event = shift;
  my $args;

  ($args = $event->{args}[0]) =~ tr/[A-Z]/[a-z]/;

  if($args =~ /^-(hani|Ѱܷ)\s*/) {
    get_hani($conn, $event, $event->nick);
  }
}

sub get_hani {
  my $conn = shift;
  my $event = shift;
  my $who = shift;
  my $args  = $event->{args}[0];

  $args =~ tr/[A-Z]/[a-z]/;
  $args =~ s/^-(hani|Ѱܷ)\s*//;
  $args =~ s/^\s*//;
  $args =~ s/\s*$//;

  my @argv = split(/ /, $args);
  my $section = '';
  my $limit = 5;

  my %SECTION = (
  '001000000'   => ['Į', ''],
  '003000000'   => ['ġ'],
  '004000000'   => [''],
  '005000000'   => ['ȸ'],
  '006000000'   => [''],
  '007000000'   => [''],
  '008000000'   => [''],
  '009000000'   => ['ȭȰ', 'ȭ', 'Ȱ'],
  '010000000'   => ['iT', ''],
  );

  if ($#argv > -1) {
    foreach my $key (keys %SECTION) {
      foreach (@{$SECTION{$key}}) {
	if (/$argv[0]/) { $section = $key; last; }
      }
      last if ($section);
    }
  }

  $limit = int($argv[$#argv]) if ($argv[$#argv] =~ /^(\d+)$/);
  $limit = 5 if ($limit <= 0);
  $limit =20 if ($limit > 20);

  if(!defined($pid = fork)) {
    $conn->privmsg($who, "error in Ѱܷ plugin...");
    return;
  }

  if($pid) {
    #parent
  
    $SIG{CHLD} = sub { wait; };
    return;
  
  } else {
    # child
  
    my ($url, $file, $http, $stat);
    if ($section ne '') {		#  Ź Ѱ跹. :p
      $url = "http://www.hani.co.kr/section-${section}/";
      $file = "${datadir}/Hani_${section}.dat";
    } else {
      $url = "http://www.hani.co.kr/TOTAL/";
      $file = "${datadir}/Hani_TOTAL.dat";
    }
    $http = new HTTP::Lite;
    $stat = $http->request($url);
    if ($stat == 200) {
	open FILE, ">" . $file;
	print FILE $http->body();
	close FILE;
    }

    open FILE, $file;

    my $p = HTML::TokeParser->new(*FILE) || die "Can't open: $!";

    if (($section ne '') && ($section ne '001000000')) {
      while(my $token = $p->get_token) {
        last if (($token->[0] eq 'C') and ($token->[1] =~ /ϴ 縮Ʈ/));
      }
    }
    while(my $token = $p->get_tag("a")) {
      if ($token->[1]{href} =~ m#/section-(\d+)/(\d+)/(\d+)/(\d+)\.html#) {
	my $tmp = $4;
	my $sec = $SECTION{substr($tmp,0,3).'000000'}[0];
	my $hour= substr($tmp, 17, 2);
	my $min = substr($tmp, 19, 2);
	my $num = substr($tmp, 21);
	my $text= $p->get_trimmed_text("/a");

	next unless ($text);

	$conn->privmsg($who, "[$sec] $text");

	last unless(--$limit);
      }
    }

    close FILE;
    $conn->{_connected} = 0;
    exit 0;
  }
}

1;
