# Joh Yong-iL <tolkien@nownuri.net>
# Date  : $Date: 2002/09/03 09:15:12 $

package Help::Plugin;

use Perlbot;
use POSIX;

sub get_hooks {
  return { public => \&on_public, msg => \&on_msg};
}

sub on_public {
  my $conn = shift;
  my $event = shift;

  if ($event->{args}[0] =~ /^-help/) {
    get_help($conn, $event, $event->{to}[0]);
  }
}

sub on_msg {
  my $conn = shift;
  my $event = shift;

  if ($event->{args}[0] =~ /^-help/) {
    get_help($conn, $event, $event->nick);
  }
}

sub get_help {
  my $conn = shift;
  my $event = shift;
  my $who = shift;

  if (!defined($pid = fork)) {
    $conn->privmsg($chan, "error in edic... weird.");
    return;
  }

  if ($pid) {
    # parent

    $SIG{CHLD} = sub { wait; };
    return;

  } else {
    # child...

    my %lookup;
    $lookup{'Ѱܷ'} = 'Hani';
    $lookup{''} = 'OhMyNews';
    $lookup{''} = 'Kma';
    $lookup{''} = 'Korail';
    $lookup{''} = 'Lunar';
    $lookup{'¥'} = 'Lunar';
    $lookup{'ȯ'} = 'Exchange';
    $lookup{'ohmy'} = 'OhMyNews';
    $lookup{''} = 'Chosun';
    $lookup{'pressian'} = 'Pressian';

    my $helpfile = new IO::File;
#    (my $in = $event->{args}[0]) =~ s/^-help\s*//;
    $event->{args}[0] =~ s/^-help\s*//;
    $event->{args}[0] =~ s/\s+/ /;
    my ($plugin, $topic) = split(' ', $event->{args}[0]);
    if ($lookup{$plugin} ne '') {
	$plugin = $lookup{$plugin};
    }

    if ($plugin) {
      if ((my $plugname) = grep(/^\Q$plugin\E$/i, @plugins)) {
	if (! $helpfile->open("<$plugindir" . $dirsep . "$plugname" . $dirsep . 'help.txt')) {
	  $conn->privmsg($who, "I can't open the help file for plugin '$plugname'");
	} else {
	  extract_help($helpfile, $plugname, $topic, $conn, $who);
	}
      } elsif (lc($plugin) eq 'core') {
	if (! $helpfile->open("<help.txt")) {
		$conn->privmsg($who, "I can't open the main help file");
	} else {
		extract_help($helpfile, 'Core', $topic, $conn, $who);
	}
      } else {
		$conn->privmsg($who, "I don't have plugin '$plugin' loaded");
      }
    } else {
      $conn->privmsg($who, "syntax: -help [plugin [topic]]");
      $conn->privmsg($who, "Returns help for the specified plugin and topic.".
       	"  If no topic is given, returns the plugin's general help message".
	" and a list of topics you can ask about.");
      $conn->privmsg($who, "PLUGINS: ".join(' ', 'Core', @plugins[1..$#plugins]));
    }

    $conn->{_connected} = 0;
    exit 0;                     # kill child
  }
}

sub extract_help {
    my ($helpfile, $plugin, $topic, $conn, $from) = @_;
    my (@help, %help, $rawtext, $basehelp);

    @help = grep(/^.+$/, $helpfile->getlines);
    chomp @help;
    $basehelp = shift(@help);
    %help = @help;
    $help{''} = $basehelp;
    unless ($rawtext = $help{$topic}) {
	$conn->privmsg($from,
		"I don't have any help for plugin '$plugin' on topic '$topic'");
    } else {
	foreach my $line (split(/\\\\/, $rawtext)) {
	    $conn->privmsg($from, $line);
	}
	if (! $topic) {
	    my @realtopics = grep(/.+/, keys(%help));
	    if (@realtopics) {
		$conn->privmsg($from, "SUBTOPICS: ".join(' ', @realtopics));
	    }
	}
    }
}

1;
