#
# @(#)IPWhere.pl
#
package IPWhere::Plugin;

use Perlbot;
use Socket;
use POSIX;

sub get_hooks {
	return { public => \&on_public, msg => \&on_msg};
}

sub on_public {
	my $conn = shift;
	my $event = shift;
	
	(my $args = $event->{args}[0]) =~ tr/[A-Z]/[a-z]/;
	
	if($args =~ /^-ipwhere\s*/) {
		funct($conn, $event, $event->{to}[0], 1);
	}
}

sub on_msg {
	my $conn = shift;
	my $event = shift;
	
	(my $args = $event->{args}[0]) =~ tr/[A-Z]/[a-z]/;

	if($args =~ /^-ipwhere\s*/) {
		funct($conn, $event, $event->nick, 0);
	}
}

# doquery(whois_server, query_name);
#
sub doquery {
	my $serv = shift;
	my $name = shift;
	my($iaddr, $paddr, $proto, $query);

	if (!defined($iaddr = inet_aton($serv))) {
		return undef;
	}
	if (!defined($paddr = sockaddr_in($port, $iaddr))) {
		return undef;
	}
	if (!defined($proto = getprotobyname("tcp"))) {
		return undef;
	}
	if (!socket(SOCK, PF_INET, SOCK_STREAM, $proto)) {
		return undef;
	}
	if (!connect(SOCK, $paddr)) {
		return undef;
	}

	$query = "$name\n";

	if (!send(SOCK, "$query", 0)) {
		return undef;
	}

	my ($buf, $org, $isp, $adr, $country, $line, $tmp);

	$line = 0;
	$org = $isp = $adr = "";

	while($buf = <SOCK>) {
		$buf =~ s/\s+$//;

		if ($serv eq 'whois.nic.or.kr') {
			#	
			if ($buf =~ /^IP ּ\s+: (.*)/) { $adr = $1; }
			elsif ($buf =~ /^ ISP\s+: (.*)/) { $isp = $1; }
			elsif ($buf =~ /^\s+: (.*)/) { $org = $1; }
			#
			elsif ($buf =~ /^1a. ѱ \s+: (.*)/) { $org = $1; }
			elsif ($buf =~ /^1b.  \s+: (.*)/) { $isp = $1; }
		}
		elsif ($serv eq 'whois.arin.net') {
			if (!$line) { $org = $buf; }
			elsif ($buf =~ /\s+Netname: (.*)/) { $isp = $1; }
			elsif ($buf =~ /\s+Netblock: (.*)/) { $adr = $1; }
		}
		else {
			if ($buf =~ /inetnum:\s+(.*)/) {
				$adr = $1 unless ($adr);
			}
			elsif ($buf =~ /netname:\s+(.*)/) {
				$isp = $1 unless($isp);
			}
			elsif ($buf =~ /descr:\s+(.*)/) {
				$tmp = $1;
				if ($tmp =~ /[a-z]/) {
					$org = $tmp unless($org);
				}
				else {
					$isp = $tmp unless($isp);
				}
			}
			elsif ($buf =~ /country:\s+(.*)/) {
				$country = $1;
			}
		}
		#
		#if ($is_debug) {
		#	print $buf."\n";
		#}
		$line++;
	}

	close (SOCK);

	my $retval;

	if ($org or $isp) {
		if ($org && $isp) {
			if ($org =~ /\((.*)\)$/) {
				if ($1 ne $isp) {
					$retval = $org =~ s/\(.*\)$/($isp)/;
				}
				else {
					$retval = $org;
				}
			}
			else {
				$retval = "$org ($isp)";
			}
		}
		elsif ($org) {
			$retval = $org;
		}
		elsif ($isp) {
			$retval = $isp;
		}
		if ($adr) {
			$retval .= "\n- IP Address: $adr";
		}
		if ($country) {
			$retval .= " [$country]";
		}
		return $retval;
	}

	return undef;
}

sub funct {
	my $conn = shift;
	my $event = shift;
	my $who = shift;
	my $limit = shift;
	my $in;
	my $task=0;
	
	if (!defined($pid = fork)) {
		$conn->privmsg($chan, "error in converting... weird.");
		return;
	}
	
	if ($pid) {
		# parent
		
		$SIG{CHLD} = sub { wait; };
		return;

	} else {
		# child...
		
		$in = $event->{args}[0];
		$in  =~ s/^-ipwhere\s*//;
		
		$in =~ s/\\//g;
		$in =~ s/\`//g;
		$in =~ s/\"//g;
		$in =~ s/\$//g;
		$in =~ s/\;//g;

		if ($in eq '') {
			$conn->{_connected} = 0;
			exit 1;
		}

		my (@whois_server, $port, $name, $addr, $arg, @argv);
		my ($is_local, $is_debug);

		$port = 43;
		$is_local = 0;
		$is_debug = 0;

		@argv = split(/\s/, $in);

		while ($arg = shift @argv) {
			if ($arg eq '-d') {
				$is_debug = 1;
			}
			elsif ($arg eq '-l') {
				$is_local = 1;
			}
			elsif ($name eq '') {
				$name = $arg;
		
				if ($name =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/) {
					$addr = $name;
				}
				else {
					if (!defined(@hostent = gethostbyname($name))) {
						$conn->{_connected} = 0;
						exit 1;
					}
					$addr = inet_ntoa($hostent[4]);
				}
			}
		}


		@whois_server = (
			'whois.ripe.net',
			'whois.apnic.net',
			'whois.ripn.net'
			);
		@whois_local = (
			'whois.nic.or.kr'
			);
		@whois_root = (
			'whois.arin.net',
			);
		
		my ($buf,$root,$serv,$pr, @outbuf);
		
		if ($addr eq $name) {
			$pr = "$name:";
		}
		else {
			$pr = "$name($addr):";
		}

		if ($is_local) {
			if ($buf = &doquery($whois_local[0], $addr)) {
				@outbuf = split(/\n/, "$pr $buf");
			}
		}
		else {
			if ($buf = &doquery($whois_local[0], $addr)) {
				@outbuf = split(/\n/, "$pr $buf");
			}
			else {
				$root = &doquery($whois_root[0], $addr);
				foreach $serv (@whois_server) {
					if ($buf = &doquery($serv, $addr)) {
						@outbuf = split(/\n/, "$pr $buf");
						last;
					}
				}
				if ($root and $buf eq '') {
					@outbuf = split(/\n/, "$pr $root");
				}
			}
		}
		foreach $buf (@outbuf) {
			$conn->privmsg($who, $buf);
		}

		$conn->{_connected} = 0;
		exit 0;                     # kill child
	}
}

1;
