# Joh Yong-iL <tolkien@nownuri.net>
# biko <pushnpop@hanmir.com>
#
# does a nslookup...
# 19990910 : -ns holywar.net - | /bin/cat /etc/passwd
#    $in =~ s/\|//;          # remove security hole
#

package Nslookup::Plugin;

use Perlbot;
use POSIX;
use Net::DNS;

sub get_hooks {
	return { public => \&on_public, msg => \&on_msg};
}

sub on_public {
	my $conn = shift;
	my $event = shift;

	(my $args = $event->{args}[0]) =~ tr/[A-Z]/[a-z]/;

	if ($args =~ /^-(nslookup|ns)/) {
		ns_lookup($conn, $event, $event->{to}[0]);
	}
}

sub on_msg {
	my $conn = shift;
	my $event = shift;

	(my $args = $event->{args}[0]) =~ tr/[A-Z]/[a-z]/;

	if ($args =~ /^-(nslookup|ns)/) {
		ns_lookup($conn, $event, $event->nick);
	}
}

sub ns_lookup {
	my $conn = shift;
	my $event = shift;
	my $who = shift;
	my $in;

	if (!defined($pid = fork)) {
		$conn->privmsg($chan, "error in nslookup... weird.");
		return;
	}

	if ($pid) { # parent
	  $SIG{CHLD} = sub { wait; };
	  return;
	} else {
	  # child...
	
	  (my $in = $event->{args}[0]) =~ s/^-(nslookup|ns)\s*//;
	  if ($in ne '') {
	    my $res = new Net::DNS::Resolver;

	    @args = split(' ', $in);
	    if ($args[0] =~ /^@/) {
		my $nameserver = shift @args;
		$nameserver =~ s/^@//;
		$res->nameservers($nameserver);
	    }

	    my ($name, $type, $class) = @args;
	    $type  ||= "A";
	    $class ||= "IN";

	    if (uc($type) ne "AXFR") {
	      my $packet = $res->send($name, $type, $class);
	      if (defined($packet)) {
		foreach my $rr ($packet->answer) {
		    if ($rr->type eq "A") {
			$conn->privmsg($who, $rr->type . ": " . $rr->address);
		    } elsif ($rr->type eq "AAAA") {
			$conn->privmsg($who, $rr->type . ": " . $rr->address);
		    } elsif ($rr->type eq "AFSDB") {
			$conn->privmsg($who, $rr->type . ": " . $rr->subtype
				. ", " . $rr->hostname);
		    } elsif ($rr->type eq "CNAME") {
			$conn->privmsg($who, $rr->type . ": " . $rr->cname);
		    } elsif ($rr->type eq "EID") {
			$conn->privmsg($who, $rr->type . ": " . $rr->rdlength
				. ", " . $rr->rdata);
		    } elsif ($rr->type eq "HINFO") {
			$conn->privmsg($who, $rr->type . ": " . $rr->cpu
				. ": " . $rr->os);
		    } elsif ($rr->type eq "ISDN") {
			$conn->privmsg($who, $rr->type . ": " . $rr->address
 				. ", " . $rr->sa);
		    } elsif ($rr->type eq "LOC") {
			$conn->privmsg($who, $rr->type . ": " . $rr->version
				. ", " . $rr->size);
			$conn->privmsg($who, "  hor: " . $rr->horiz_pre
				. "ver: " . $rr->vert_pre
				. "lat: " . $rr->latitude
				. "lon: " . $rr->longitude
				. "latlon: " . $rr->latlon
				. "alt: " . $rr->altitude);
		    } elsif ($rr->type eq "MB") {
			$conn->privmsg($who, $rr->type . ": " . $rr->madname);
		    } elsif ($rr->type eq "MG") {
			$conn->privmsg($who, $rr->type . ": " . $rr->mgmname);
		    } elsif ($rr->type eq "MINFO") {
			$conn->privmsg($who, $rr->type
				. ": rmail " . $rr->rmailbx
				. ", email " . $rr->emailbx);
		    } elsif ($rr->type eq "MR") {
		      	$conn->privmsg($who, $rr->type . ": " . $rr->newname);
		    } elsif ($rr->type eq "MX") {
			$conn->privmsg($who, $rr->type
				. ": pref " . $rr->preference
				. ", ex " . $rr->exchange);
		    } elsif ($rr->type eq "NAPTR") {
			$conn->privmsg($who, $rr->type . ": ord " . $rr->order
				. ", pref " . $rr->preference
				. ", fl " . $rr->flags);
			$conn->privmsg($who, "  svc " . $rr->service
				. ", reg " . $rr->regexp
				. ", rep " . $rr->replacement);
		    } elsif ($rr->type eq "NIMLOC") {
			$conn->privmsg($who, $rr->type . ": " . $rr->rdlength
				. ", " . $rr->rdata);
		    } elsif ($rr->type eq "NS") {
			$conn->privmsg($who, $rr->type . ": " . $rr->nsdname);
		    } elsif ($rr->type eq "NSAP") {
			$conn->privmsg($who, $rr->type . ": idp " . $rr->idp
				. ", dsp " . $rr->dsp
				. ", afi " . $rr->afi
				. ", aa " . $rr->aa);
			$conn->privmsg($who, "  rsvd " . $rr->rsvd
				. ", rd " . $rr->rd
				. ", area " . $rr->area
				. ", id " . $rr->id
				. ", sel " . $rr->sel);
		    } elsif ($rr->type eq "NULL") {
			$conn->privmsg($who, $rr->type . ": " . $rr->rdlength
				. ", " . $rr->rdata);
		    } elsif ($rr->type eq "PTR") {
		      	$conn->privmsg($who, $rr->type . ": " . $rr->ptrdname);
		    } elsif ($rr->type eq "PX") {
		      	$conn->privmsg($who, $rr->type
				. ": pref " . $rr->preference
				. ", 822 " . $rr->map822
				. ", 400 " . $rr->map400);
		    } elsif ($rr->type eq "RP") {
			$conn->privmsg($who, $rr->type . ": mbox " . $rr->mbox
				. ", txt " . $rr->txtdname);
		    } elsif ($rr->type eq "RT") {
			$conn->privmsg($who, $rr->type
				. ": pref " . $rr->preference
				. ", itmd " . $rr->intermediate);
		    } elsif ($rr->type eq "SOA") {
			$conn->privmsg($who, $rr->type . ": " . $rr->mname
				. ", rname " . $rr->rname
				. ", ser " . $rr->serial);
			$conn->privmsg($who, $rr->type
				. "  ref " . $rr->refresh
				. ", ret " . $rr->retry
				. ", exp " . $rr->expire
				. ", min " . $rr->minimum);
		    } elsif ($rr->type eq "SRV") {
		      	$conn->privmsg($who, $rr->type
				. ": pri " . $rr->priority
				. ", wei " . $rr->weight
				. ", prt " . $rr->port
				. ", tgt " . $rr->target);
		    } elsif ($rr->type eq "TXT") {
			$conn->privmsg($who, $rr->type . ": " . $rr->txtdata);
		    } elsif ($rr->type eq "X25") {
			$conn->privmsg($who, $rr->type . ": " . $rr->psdn);
		    }
		}
	      } else {
		$conn->privmsg($who, "query failed: " . $res->errorstring);
	      }
	    }
	  }
	$conn->{_connected} = 0;
	exit 0;                     # kill child

	} # end of child
}

1;
