#
# @(#)SetiAtHome.pl
#
package SetiAtHome::Plugin;

use Perlbot;
use Socket;
use POSIX;

sub get_hooks {
	return { public => \&on_public, msg => \&on_msg};
}

sub on_public {
	my $conn = shift;
	my $event = shift;
	
	(my $args = $event->{args}[0]) =~ tr/[A-Z]/[a-z]/;
	
	if($args =~ /^-seti\s*/) {
		funct($conn, $event, $event->{to}[0], 1);
	}
}

sub on_msg {
	my $conn = shift;
	my $event = shift;
	
	(my $args = $event->{args}[0]) =~ tr/[A-Z]/[a-z]/;

	if($args =~ /^-seti\s*/) {
		funct($conn, $event, $event->nick, 0);
	}
}

sub funct {
	my $conn = shift;
	my $event = shift;
	my $who = shift;
	my $limit = shift;
	my $in;
	my $task=0;
	
	if (!defined($pid = fork)) {
		$conn->privmsg($chan, "error in converting... weird.");
		return;
	}
	
	if ($pid) {
		# parent
		
		$SIG{CHLD} = sub { wait; };
		return;

	} else {
		# child...
		
		$in = $event->{args}[0];
		$in  =~ s/^-seti\s*//;
		
		$in =~ s/\\//g;
		$in =~ s/\`//g;
		$in =~ s/\"//g;
		$in =~ s/\$//g;
		$in =~ s/\;//g;

		if ($in eq '') {
			$in = "SarangNET";
		}

		my($serv, $port, $iaddr, $paddr, $proto, $query, $is_user);

		$is_user = 0;
		if ($in =~ /@/) {
			$is_user = 1;
		}

		$serv = "setiathome.ssl.berkeley.edu";
		$port = 80;

		if (!defined($iaddr = inet_aton($serv))) {
			$conn->{_connected} = 0;
			exit 1;
		}
		if (!defined($paddr = sockaddr_in($port, $iaddr))) {
			$conn->{_connected} = 0;
			exit 1;
		}
		if (!defined($proto = getprotobyname("tcp"))) {
			$conn->{_connected} = 0;
			exit 1;
		}
		if (!socket(SOCK, PF_INET, SOCK_STREAM, $proto)) {
			$conn->{_connected} = 0;
			exit 1;
		}
		if (!connect(SOCK, $paddr)) {
			$conn->{_connected} = 0;
			exit 1;
		}

		if ($is_user) {
			$query = "GET /cgi-bin/cgi?cmd=user_stats_new&email=${in} HTTP/1.0";
		}
		else {
			$query = "GET /cgi-bin/cgi?cmd=team_lookup&name=$in HTTP/1.0";
		}
		if (!send(SOCK, "$query\n\n", 0)) {
			close(SOCK);
			$conn->{_connected} = 0;
			exit 1;
		}

		# parse

		if ($is_user) {
			my ($buf, $result, $cputime, $rank);
			while($buf = <SOCK>) {
				if ($buf =~ /Results Received<\/TD><TD>(\d+)</) {
					$result = $1;
				}
				elsif ($buf =~ /Total CPU Time<\/TD><TD>\s+(\d+[^<]+)</) {
					$cputime = $1;
				}
				elsif ($buf =~ /^<TR><TD>Your rank out of/) {
					my ($usr, $tot);
					if ($buf =~ /<b>(\d+)<\/b>/) { $tot = $1; }
					if ($buf =~ /<b>(\d+)<sup>/) { $usr = $1; }
					$rank = "$usr/$tot";
				}
			}
			$conn->privmsg($who, "$in: Result: $result / Total CPU time: $cputime / Rank: $rank");
		}
		else {
			my ($buf, $members, $results, $cputime, @topmember, $users, $tmp);
			$users = 0;
			while($buf = <SOCK>) {
				if ($buf =~ /Members/) {
					if ($buf =~ /(\d+)</) {
						$members = $1;
					}
				}
				elsif ($buf =~ /Results received/) {
					if ($buf =~ /(\d+)</) {
						$results = $1;
					}
				}
				elsif ($buf =~ /Total CPU time<\/td><td>\s+(\d+[^<]+)</) {
					$cputime = $1;
				}
				elsif ($buf =~ /^<tr><td>(\d\)) ([^<]+)<.*><.*>(\d+)</) {
					my ($a, $b, $c);
					$c = $3;
					$a = $1;
					$b = $2; $b =~ s/&nbsp;//;
					$tmp = sprintf(" %-23s: %5d", "$a $b", $c);
					if ($buf = <SOCK>) {
						if ($buf =~ /(\d+[^<]+)/) { $tmp .= " / $1"; }
					}
					if ($buf = <SOCK>) {
						if ($buf =~ /(\d+[^<]+)/) { $tmp .= " / $1"; }
					}
					push(@topmember, $tmp);
					if (++$users >= $limit) {
						last;
					}
				}
				elsif ($buf =~ /^<tr><td>(\d\)) <a href=.*>/) {
					my $a = $1;
					$buf = <SOCK>;
					$buf =~ s/\s*$//;
					$tmp = sprintf(" %-23s: ", "$a $buf");
					if ($buf = <SOCK>) {
						if ($buf =~ /(\d+[^<]+)/) { $tmp .= sprintf("%5d", $1); }
					}
					if ($buf = <SOCK>) {
						if ($buf =~ /(\d+[^<]+)/) { $tmp .= " / $1"; }
					}
					if ($buf = <SOCK>) {
						if ($buf =~ /(\d+[^<]+)/) { $tmp .= " / $1"; }
					}
					push(@topmember, $tmp);
					if (++$users >= $limit) {
						last;
					}
				}
			}
			$conn->privmsg($who, "[$in] Members: $members / Results: $results / CPU Time: $cputime");
			foreach $tmp (@topmember) {
				$conn->privmsg($who, $tmp);
			}
		}

		$conn->{_connected} = 0;
		exit 0;                     # kill child
	}
}

1;
